package com.BobCo.FileExplorer;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import com.tivoli.twg.engine.TWGConObject;
import com.tivoli.twg.engine.TWGConManagedObject;
import com.tivoli.twg.libs.IntelByteBuffer;

/**
 * This class extends DefaultMutableTreeNode and provides dynamic
 * update of the tree by issuing commands to the client as branches
 * are expanded.
 */
public class BobCoFileExplorerTreeNode extends DefaultMutableTreeNode {
   public static final int ROOT_MO   = 1;
   public static final int DRIVE     = 2;
   public static final int DIRECTORY = 3;
   public static final int FILE      = 4;

   private int type = FILE;

   private String name;
   private String fullPath;
   private boolean hasLoaded = false;
   private BobCoFileExplorerGUI gui = null;
   private TWGConManagedObject cmo = null;

   /**
    * Constructor for managed object node
    * @param gui   the BobCoFileExplorerGUI instance that owns the tree
    * @param moid  the managed object ID of the client to explore
    */
   BobCoFileExplorerTreeNode(BobCoFileExplorerGUI gui, long moid) {
      this.gui = gui;
      cmo = (TWGConManagedObject)TWGConObject.FindObject(moid);
      name = cmo.getName();
      type = ROOT_MO;
   }

   /**
    * Constructor for drive node
    * @param gui          the BobCoFileExplorerGUI instance that owns the tree
    * @param driveLetter  the drive letter identifier (i.e. "C:")
    */
   BobCoFileExplorerTreeNode(BobCoFileExplorerGUI gui, String driveLetter) {
      this.gui = gui;
      name = driveLetter;
      fullPath = driveLetter + "*";
      type = DRIVE;
   }

   /**
    * Constructor for directory or file
    * @param gui         the BobCoFileExplorerGUI instance that owns the tree
    * @param parentPath  the fully qualified path of the directory containing
    *                    the new directory or file (i.e. "C:\winnt\*"
    * @param name        the name of the new file or directory
    * @param isDir       boolean, true indicates that new node is a directory
    */
   BobCoFileExplorerTreeNode(BobCoFileExplorerGUI gui, String parentPath, String name, boolean isDir) {
      this.gui = gui;
      this.name = name;
      StringBuffer sb = new StringBuffer(parentPath);
      sb.insert(sb.length()-2, '\\');
      sb.insert(sb.length()-2, name);
      fullPath = sb.toString();
      type = isDir ? DIRECTORY : FILE;
   }

   /**
    * Override getAllowsChildren() so that it returns false only for
    * file nodes.
    */
   public boolean getAllowsChildren() {
      return (type != FILE);
   }

   /**
    * Override getChildCount() so that it calls loadChildren() the
    * first time a drive or directory node is opened.
    */
   public int getChildCount() {
      if (!hasLoaded) {
         loadChildren();
      }
      return super.getChildCount();
   }

   /**
    * Based on the node type, send an asynchronous command to the
    * client to get the list of drives or the contents or a drive
    * or directory to populate the children of a managed object,
    * drive or directory node.
    */
   protected void loadChildren() {
      BobCoFileExplorerTreeNode newNode;
      int i = 0;

      gui.startStatusIndicator();
      gui.setStatusText("Retrieving data from client...");
      switch (type) {
         case ROOT_MO:    // send command to get drives
            {
               ConsoleRequestLocalFixedDrivesCommand getDrives = 
			      new ConsoleRequestLocalFixedDrivesCommand(this, gui.getTaskActivator().getMoid().GetValue(0));
               if (!gui.sendAsynchToServer(getDrives)) {
                  System.out.println("*** BobCo GUI error: Request for fixed drives failed.");
                  gui.setStatusText("Ready");
                  gui.stopStatusIndicator();
               }
            }
            break;

         case DRIVE:
         case DIRECTORY:  // send command to get directory contents
            {
               ConsoleRequestDirectoryContentsCommand getDir =
			      new ConsoleRequestDirectoryContentsCommand(this, gui.getTaskActivator().getMoid().GetValue(0), fullPath);
               if (!gui.sendAsynchToServer(getDir)) {
                  System.out.println("*** BobCo GUI error: Request for directory contents failed.");
                  gui.setStatusText("Ready");
                  gui.stopStatusIndicator();
               }
            }
            break;

         case FILE:       // shouldn't get this case, but if so, do nothing
         default:
            break;
      }
      hasLoaded = true;
   }

   /**
    * Overrides get user object in order to allow this object to be
    * used properly by the cell renderer
    */
   public Object getUserObject () {
      return this;
   }

   /**
    * Returns the type of this node.
    */
   public int getType() {
      return type;
   }

   /**
    * Get the name of this node for display.
    */
   public String getName() {
      return name;
   }

   /**
    * Get the console managed object that this node is for.
    */
   public TWGConManagedObject getConManagedObject() {
      return cmo;
   }

   /**
    * Command completion processing code for a fixed drive query command.
    * It loops through the output parameters of the command an adds a new
    * drive node to the tree for each one.
    */
   public void localFixedDrivesCommandComplete(ConsoleRequestLocalFixedDrivesCommand cmd) {
      BobCoFileExplorerTreeNode newNode = null;
      int[] childIndices = null;

      if (cmd.ReturnCode() == BobCoFileExplorerConstants.RETURN_OKAY) {
         System.out.println("Adding drives, count="+cmd.NumOutputParms());
         childIndices = new int[cmd.NumOutputParms()];
         for (int i=0; i<childIndices.length; i++) {
            childIndices[i] = i;
            newNode = new BobCoFileExplorerTreeNode(gui, IntelByteBuffer.ReadCompUnicode(cmd.OutputParm(i),0));
            insert(newNode, i);
         }
         if (childIndices != null) {
            ((DefaultTreeModel)gui.getTree().getModel()).nodesWereInserted(this, childIndices);
         }
      } else {
         System.out.println("*** BobCo GUI error: Request for fixed drives failed, rc="+cmd.ReturnCode());
      }
      gui.setStatusText("Ready");
      gui.stopStatusIndicator();
   }

   /**
    * Command completion processing code for a directory contents query command.
    * It loops through the output parameters of the command an adds a new
    * directory or file node to the tree for each one.
    */
   public void directoryContentsCommandComplete(ConsoleRequestDirectoryContentsCommand cmd) {
      BobCoFileExplorerTreeNode newNode = null;
      int[] childIndices = null;

      if (cmd.ReturnCode() == BobCoFileExplorerConstants.RETURN_OKAY) {
         System.out.println("Adding directories/files, count="+cmd.NumOutputParms());
         IntelByteBuffer ibb = new IntelByteBuffer(0);
         childIndices = new int[cmd.NumOutputParms()];
         for (int i=0; i<childIndices.length; i++) {
            childIndices[i] = i;
            ibb.SetBuffer(cmd.OutputParm(i));
            ibb.ResetStart(0);
            int    fileAttributes = (int)ibb.ReadULONG();
            long   fileSize       = ibb.ReadLong64();
            int    dosDate        = (int)ibb.ReadUSHORT();
            int    dosTime        = (int)ibb.ReadUSHORT();
            String fileName       = ibb.ReadCompUnicode();
            newNode = new BobCoFileExplorerTreeNode(gui, fullPath, fileName, ((fileAttributes & BobCoFileExplorerConstants.DIRECTORY) != 0));
            insert(newNode, i);
         }
         if (childIndices != null) {
            ((DefaultTreeModel)gui.getTree().getModel()).nodesWereInserted(this, childIndices);
         }
      } else {
         System.out.println("*** BobCo GUI error: Request for directory contents failed, rc="+cmd.ReturnCode());
      }
      gui.setStatusText("Ready");
      gui.stopStatusIndicator();
   }
}
