/***************************************************************************
 * IBM Confidential                                                        *
 *                                                                         *
 * OCO Source Materials                                                    *
 *                                                                         *
 * program number (04L8267)                                                *
 *                                                                         *
 * (C) Copyright 1998 by IBM                                               *
 *                                                                         *
 *                                                                         *
 * The source code for this program is not published or otherwise divested *
 * of its trade secrets, irrespective of what has been deposited with the  *
 * U.S. Copyright Office.                                                  *
 ***************************************************************************/

package com.BobCo.associations;

import com.tivoli.twg.log.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.inventory.TWGWindowsNetworkIdTable;
import com.tivoli.twg.log.TWGRas;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;

import java.util.*;

/**
 * Sample association subclass for supporting Domain/Workgroup topology based 
 * relationships.  This sample demonstrates the base functionality required to
 * implement a new association for Director managed object presentation.
 * <p>For this sample, the data used for defining the relationships of the association
 * are based on the "DOMAIN" and "WORKGROUP" fields of the "TWG_WINDOWS_NETWORK_ID"
 * table in the Director Database.  This data describes the name of the 
 * Windows NT Domain that each system is a member of (if its a domain system) and
 * the Windows Workgroup the system is a member of (if its a workgroup system).
 * For each different domain and/or workgroup name returned for each of the
 * systems passed to the association, a <code>TWGRelationship</code> object is
 * defined, and named according to the domain or workgroup name.  Each system which
 * is a member of a given domain or workgroup will then be <code>down-linked</code>
 * below the <code>TWGRelationship</code> instance corresponding to its domain/
 * workgroup.  
 * <p>Also, note that all the domain and workgroup <code>TWGRelationship
 * </code> instances are linked to a <code>root</code> relationship.  This
 * <code>root</code> relationship is not displayed as a folder by the 
 * Director console, but is required for the other <code>TWGRelationships</code>
 * to be displayed properly.
 * <p>Also of note is the full internationalization enabling of this association
 * implementation.  When a <code>TWGRelationship</code> is queried for its name
 * for presentation on the Director console, the <code>Locale</code>
 * of the console is provided as input, allowing appropriate strings and formatting
 * to be done.  The <code>TWGRelationship</code> base class provides setting of
 * a reference to a static string resource in a <code>java.util.ResourceBundle</code>,
 * but for variable data (such as our domain and workgroup names), a simple
 * subclass of <code>TWGRelationship</code> (the <code>DomainRelationship</code>
 * inner class shown below) can be implemented which provides a custom
 * implementation of the <code>getName(Locale)</code> method.  In our case, the
 * <code>java.text.MessageFormat</code> class is used to produce a language-specific
 * string corresponding to "Domain 'name'" or "Workgroup 'name'".
 * 
 * @see com.tivoli.twg.engine.TWGAssociation
 * @see com.tivoli.twg.engine.TWGDatabaseTable
 * @see com.tivoli.twg.engine.TWGRelationship
 * @see com.tivoli.twg.engine.TWGRelationshipSet
 * @see com.tivoli.twg.engine.TWGManagedObjectSet
 * @see com.tivoli.twg.engine.TWGManagedObject
 * 
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class DomainWorkgroupAssociation extends TWGAssociation {
    /** ID of default "Domains/Workgroups" association                             */
    public static final String DEFAULT_DOMAIN_ASSOC_ID = 
	"com.BobCo.domain_workgroup";
    /** Name ID of default "Domains/Workgroups" association                        */
    private static final String DEFAULT_DOMAIN_ASSOC_NAME_ID = 
	"assoc.domain.name";
    /** Name ID of "Domain 'xx'" format string                                     */
    private static final String DOMAIN_NAME_ID = 
	"assoc.domain.domainlabel";
    /** Name ID of "Workgroup 'xx'" format string                                  */
    private static final String WORKGROUP_NAME_ID = 
	"assoc.domain.workgrouplabel";
    /** NLS bundle of default "Domains/Workgroups" association                     */
    static final String DEFAULT_DOMAIN_ASSOC_NLS_BUNDLE = 
	"com.BobCo.associations.DomainWorkgroupNLS";   
    /** Root relationship ID                                                       */
    private static final String ROOT_REL_ID = "root";
    /* Column indexes for our query                                                */
    private static final int MANAGED_OBJ_ID_COL = 1;
    private static final int COMPUTER_NAME_COL = 2;
    private static final int DOMAIN_COL = 3;
    private static final int WORKGROUP_COL = 4;
    /**
     * Default constructor for sample Domain/Workgroups topology association
     */
    public DomainWorkgroupAssociation() {
	super();
    }
    /**
     * Initializer method : used to build default instance, if needed
     * @exception <code>TWGPersistentObjectSaveException</code> if error
     *    saving new objects
     */
    public static void initializeDefault() throws
	TWGPersistentObjectSaveException {
	/* If default 'Domains/Workgroups' association not found, build it      */
	if(findByIDString(DEFAULT_DOMAIN_ASSOC_ID) == null) {
	    DomainWorkgroupAssociation assoc = new DomainWorkgroupAssociation();
	    assoc.setName(DEFAULT_DOMAIN_ASSOC_NAME_ID,
		DEFAULT_DOMAIN_ASSOC_NLS_BUNDLE);
	    assoc.setIDString(DEFAULT_DOMAIN_ASSOC_ID);
	    assoc.setMenuWeight(0);	/* Use default menu weight		*/
	    assoc.save();
	}
    }
    /**
     * Primary functional evaulation method for association object.
     * This method creates a relationship object for each distinct
     * workgroup and domain, containing the systems under each domain and
     * workgroup.
     * @param rel - Relationship set to be maintainted
     * @param add_set - Set of persistent IDs for the managed objects which
     *     have been added to the TWGManagedObjectSet which was used to
     *     produce the relationship set
     * @param chg_set - Set of persistent IDs for the managed objects which
     *     have been changed (and were previously in the MOS used for the
     *     relationship set).
     * @param mos - TWGManagedObjectSet containing full population of objects
     *     for relationship set (with changes already applied)
     */
    public void evaluateNewAndChangedObjects(TWGRelationshipSet rel,
	IntValueSet add_set, IntValueSet chg_set, TWGManagedObjectSet mos) {
        TWGRelationship base_r = null;
        /* Find and add base relationship                                   */
        base_r = rel.findRelationshipByID(ROOT_REL_ID);
        /* If not found, create it                                          */
        if(base_r == null) {
            try {
                base_r = new TWGRelationship(null, rel, ROOT_REL_ID);
            } catch (InvalidObjectIDException ioidx) {
                return;
            }
            /* This name isn't presented, so no translation needed          */
            base_r.setName("All Domain/Workgroup systems");
        }
                                       
	// vector to contain a query field object for each selected column in the query
	Vector selectFields = new Vector();

	//Add columns to query for use by TWG_DB_CREATE_RUN_QUERY_CMD Command
	selectFields.addElement(new TWGDbQueryField(TWGWindowsNetworkIdTable.TABLE_NAME,TWGWindowsNetworkIdTable.MANAGED_OBJ_ID));
        selectFields.addElement(new TWGDbQueryField(TWGWindowsNetworkIdTable.TABLE_NAME,TWGWindowsNetworkIdTable.COMPUTER_NAME));
        selectFields.addElement(new TWGDbQueryField(TWGWindowsNetworkIdTable.TABLE_NAME,TWGWindowsNetworkIdTable.DOMAIN));
        selectFields.addElement(new TWGDbQueryField(TWGWindowsNetworkIdTable.TABLE_NAME,TWGWindowsNetworkIdTable.WORKGROUP));

	//Get result set from TWG_DB_CREATE_RUN_QUERY_CMD via RunSimpleQuery static class function
	TWGDbResultSet rs = RunSimpleQuery.RunQuery(selectFields);

        try{
            /* Remove any changed objects : we'll add them again later, if   */
            /* they're still supposed to be present                          */
            for(int i = 0; i < chg_set.Length(); i++) {
	        TWGManagedObject mo = TWGManagedObject.getManagedObjectByID(
		    chg_set.GetValue(i));
                if(mo != null) {    /* If object found, get IPX addresses    */
		    rel.removeAllLinksToManagedObject(mo);
                }
            } 

            /* While we have more results, find objects to add relationships */
            /* If no results, no objects will be added                       */
            if(rs == null) {
                return;
            }
	    while(rs.next()){
                int obj_id = (int)rs.getInt(MANAGED_OBJ_ID_COL); /* Get Object ID */
                /* If not an ID we're interested in, skip it                 */
                if((add_set.Find(obj_id) == -1) && 
                   (chg_set.Find(obj_id) == -1)) {
                    continue;
                }
		
	        TWGManagedObject mo = TWGManagedObject.getManagedObjectByID(
                    obj_id);
                if(mo == null) {
                    continue;
                }
                /* Get machine name                                          */
                String mach_name = rs.getString(COMPUTER_NAME_COL).trim();	
                /* Get domain name                                           */
                String domain_name = rs.getString(DOMAIN_COL).trim();	
                /* Get wgroup name                                           */
                String workgroup_name = rs.getString(WORKGROUP_COL).trim(); 
                /* If domain name is non-null and not equal to the machine   */
                /* name, add a domain relationship                           */
                if((domain_name != null) && (domain_name.length() != 0) &&
                    (!domain_name.equals(mach_name))) {
                    /* Find corresponding domain relationship                */
                    String domrelname = "DOM:" + domain_name;
                    DomainRelationship r = (DomainRelationship)rel.
                        findRelationshipByID(domrelname);
                    try {
                        /* If base isn't defined, add it                     */
                        if(r == null) {
                            r = new DomainRelationship(DOMAIN_NAME_ID,
                                domain_name, base_r, rel, domrelname);
                        }
                        /* Add ourself to the domain relationship            */
                        r.AddDownLink(mo);
                    } catch (InvalidObjectIDException ioidx) {
                    }
                }
                /* If workgroup name is not null, add workgroup relationship */
                if((workgroup_name != null) && (workgroup_name.length() != 0)) {
                    /* Find corresponding workgroup relationship             */
                    String wgrelname = "WG:" + workgroup_name;
                    DomainRelationship r = (DomainRelationship)rel.
                        findRelationshipByID(wgrelname);
                    try {
                        /* If base isn't defined, add it                     */
                        if(r == null) {
                            r = new DomainRelationship(WORKGROUP_NAME_ID,
                                workgroup_name, base_r, rel, wgrelname);
                        }
                        /* Add ourself to the workgroup relationship         */
                        r.AddDownLink(mo);
                    } catch (InvalidObjectIDException ioidx) {
                    }
                }
            }
        } catch (TWGDbException dbx) {
            String err = "DomainWorkgroupAssociation: createStatement error";
            TWGRas.error(TWGRas.ENGINE, err , dbx);
        } finally {
            if(rs != null) {
            }
	}
    }
    /**
     * Private subclass for our relationships : adds support for localized
     * translations of "Domain 'x'" and "Workgroup 'y'" labels
     */
    private static class DomainRelationship extends TWGRelationship {
        /* Template name for label                                           */
        private String template_name;	
        /* Value of name (for label)                                         */
        private String our_label;
        /**
         * Constructor for relationship
         * @param tmplt - template name
         * @param lbl - value for name
         * @param parent - Parent (<code>primary link</code>") 
         *    <code>TWGRelationship</code>, or null if none
         * @param rel_set - <code>TWGRelationshipSet</code> to contain object
         * @param relationship_id - ID string of relationship object
         * @exception <code>InvalidObjectIDException</code> if invalid object
         */
        private DomainRelationship(String tmplt, String lbl,
            TWGRelationship parent, TWGRelationshipSet rel_set, 
            String relationship_id) throws InvalidObjectIDException {
            super(parent, rel_set, relationship_id);
            template_name = tmplt;
            our_label = lbl.trim();
        }
        /**
         * Get name of filter (using given locale)
         * @param loc - locale of request
         * @return name
         */
        public String getName(Locale loc) {
            try {
                MultiLocaleBundle mlb = 
                    MultiLocaleBundle.getSharedBundle(DEFAULT_DOMAIN_ASSOC_NLS_BUNDLE);
                String fmt = mlb.getString(loc, template_name);
                Object[] args = { our_label };
                return MessageFormat.format(fmt, args);
            } catch (MissingResourceException mrx) {
            }
            return our_label;
        }
    }
}
