/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 2003.  All Rights Reserved.
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.TWGFilter;
import com.tivoli.twg.engine.TWGFilterException;
import com.tivoli.twg.engine.TWGFilterParms;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectConstants;
import com.tivoli.twg.engine.TWGManagedObjectSet;
import com.tivoli.twg.engine.TWGObjectDestroyException;
import com.tivoli.twg.engine.TWGPersistentObject;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.TWGString;
import com.tivoli.twg.libs.IntValueSet;
import com.tivoli.twg.libs.MultiLocaleBundle;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.engine.cli.ICliBundle;
import java.io.Writer;
import java.io.IOException;
import java.math.BigInteger;
import java.text.MessageFormat;
import java.util.*;

/**
 * This command line interface (CLI) example shows how to create a
 * bundle extending the command line framework.  The steps to creating
 * a bundle are:<p>
 *
 * <p>(1) implement the ICliBundle interface
 * <p>(2) use the below method signature for all accessible bundle commands:
 * <li> public int commandName(Locale locale, String[] args, Writer out) throws Exception
 * <p>(3) furnish the help method
 *
 * <p> The name of the method is used as the name of the command and this is not re-definable.
 * <p> When a user executes a command the corresponding method is invoked where:
 *           <li> Locale = client locale
 *           <li> args   = command inputs provided by the user
 *           <li> out    = output stream back to the client command shell
 *
 * <p>This particular bundle example implements two usage/help commands:
 * <li> Help - Displays written help message to the user
 * <li> List -  Shows the command syntax
 *
 * <p>Two functional commands are implemented as well.  One to list all of the subagents
 * for a specified set of systems, and the other to verify a list of service nodes.
 * The commands are below:
 * <li> ListSystemSubAgents
 * <li> CheckServiceNodes
 *
 * @see com.tivoli.twg.engine.cli.ICliBundle
 *
 * @version 1.0 08/18/03
 */
public class SDKCLI implements ICliBundle
{
  private MultiLocaleBundle mlb = null;
  private MultiLocaleBundle engine_mlb = null;
  /* Name of our Resource Bundle class */
  private static final String OURRESOURCEBUNDLE = "com.ibm.sysmgt.sdk.tools.SDKCLIResourceBundle";
  private static final String ENGINERESOURCEBUNDLE = "com.tivoli.twg.engine.EngineNLS";

  private static String[] state_ids = {
        "state.lbl.unknown", "state.lbl.indeterminate",
        "state.lbl.err_offline", "state.lbl.offline",
        "state.lbl.err_online", "state.lbl.online",
        "state.lbl.unlicensed" };

  /**
   * Default and only constructor<p>
   * All CLI bundle extensions must have an empty parameter constructor.
   * The constructor can either be omitted or defined with no parameters
   */
  public SDKCLI()
  {
    try
    {
      mlb = MultiLocaleBundle.getSharedBundle(OURRESOURCEBUNDLE);
      engine_mlb = MultiLocaleBundle.getSharedBundle(ENGINERESOURCEBUNDLE);
    }
    catch (MissingResourceException mrx)
    {
      mlb = null;
    }
  }

  /**
   * Mandated bundle command to provide bundle help
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (nothing expected for this command)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int help(Locale locale, String[] args, Writer out) throws Exception
  {
    out.write(getString("USAGE.1"));   // This is the IBM Director SDK Command Line Interface.\n
    out.write(getString("USAGE.2"));   // Listed below are the Command Line Interface commands:\n
    out.write(getString("USAGE.3"));   //   Help - Shows current help screen\n
    out.write(getString("USAGE.4"));   //   List - List the bundle commands and syntax\n
    out.write(getString("USAGE.5"));   //   CreateStaticGroup - Create a group
    out.write(getString("USAGE.6"));   //   DeleteStaticGroup - Delete a group
    out.write(getString("USAGE.7"));   //   CreateSystem - Create a managed system
    out.write(getString("USAGE.8"));   //   DeleteSystem - Delete a managed system
    out.write(getString("USAGE.9"));   //   AddSystemToGroup - Add an managed system to a group
    out.write(getString("USAGE.10"));  //   RemoveSystemFromGroup - Remove an managed system from a group
    out.write(getString("USAGE.18"));  //   ListSystemsInGroup - List the managed systems in a group
    out.write(getString("USAGE.11"));  //   AddAttribute - Add an attribute to an managed system
    out.write(getString("USAGE.12"));  //   RemoveAttribute - Remove an attribute from an managed system
    out.write(getString("USAGE.19"));  //   ListAttributes - List the attributes of a managed system
    out.write(getString("USAGE.20"));  //   ListUserDefinedAttributes - List the user defined attributes of a user defined managed system
    out.write(getString("USAGE.21"));  //   ListSystemByUserDefinedAttribute - List user defined managed systems by a user defined attribute
    out.write(getString("USAGE.22"));  //   ChangeUserDefinedAttributeValue - Sets the subtype attribute for a user defined managed system
//    out.write(getString("USAGE.13"));  //   CreateTask - Create a task
//    out.write(getString("USAGE.14"));  //   DeleteTask - Delete a task
//    out.write(getString("USAGE.15"));  //     RunTask - Run a task
    out.write(getString("USAGE.16"));  //     GetStatus - Get the status of a managed system.
    out.write(getString("USAGE.17"));  //     SetStatus - Set the status of a managed system.
    return ICliBundle.USAGE;
  }

  /**
   * Bundle command to provide bundle a short/quick command reference
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (nothing expected for this command)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int list(Locale locale, String[] args, Writer out) throws Exception
  {
    out.write(getString("LIST.1"));   // Command list\n
    out.write(getString("LIST.2"));   //   Help\n
    out.write(getString("LIST.3"));   //   List\n
    out.write(getString("LIST.4"));   //   CreateStaticGroup <group>
    out.write(getString("LIST.5"));   //   DeleteStaticGroup <group>
    out.write(getString("LIST.6"));   //   CreateSystem <systemname>
    out.write(getString("LIST.7"));   //   DeleteSystem <systemname>
    out.write(getString("LIST.8"));   //   AddSystemToGroup <group> <systemname>
    out.write(getString("LIST.9"));   //   RemoveSystemFromGroup <group> <systemname>
    out.write(getString("LIST.17"));  //   ListSystemsInGroup <group>
    out.write(getString("LIST.10"));  //   AddAttribute <attribute> <description> <value> <systemname>
    out.write(getString("LIST.11"));  //   RemoveAttribute <attribute> <systemname>
    out.write(getString("LIST.18"));  //   ListAttributes <systemname>
    out.write(getString("LIST.19"));  //   ListUserDefinedAttributes <systemname>
    out.write(getString("LIST.20"));  //   ListSystemByUserDefinedAttribute <attribute>=<value> <systemname>
    out.write(getString("LIST.21"));  //   ChangeUserDefinedAttributeValue <attribute>=<value> <systemname>
//    out.write(getString("LIST.12"));  //   CreateTask <taskname> <path>
//    out.write(getString("LIST.13"));  //   DeleteTask <taskname>
//    out.write(getString("LIST.14"));  //   RunTask <taskname> <systemname>
    out.write(getString("LIST.15"));  //   GetStatus <systemname>
    out.write(getString("LIST.16"));  //   SetStatus <status> <systemname>
    return ICliBundle.USAGE;
  }

  /**
   * Create a group
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int createStaticGroup(Locale locale, String[] args, Writer out) throws Exception
  {
    int rc = ICliBundle.OK;

    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.createStaticGroup()\n");

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_GROUP"));
      return ICliBundle.USAGE;
    }

    for (int i = 0; i < args.length; i++)
    {
      // verify that another group with the same name
      // doesn't already exist
      if (TWGFilter.findFilterByName(args[i]) != null)
      {
        out.write(MessageFormat.format(getString("ERROR.GROUP_DUPLICATE"),
                                 new Object[] {args[i]}));
        rc = ICliBundle.FAIL;
      }
      else
      {
        // attempt to create the group
        try
        {
          TWGFilterParms parms = new TWGFilterParms(args[i], new IntValueSet());
          TWGFilter filterObject = new TWGFilter(parms);
          filterObject.save();
        }
        catch(TWGFilterException e)
        {
          //TWGRas.error(TWGRas.ENGINE, "SDKCLI.createStaticGroup " + e);
          out.write(MessageFormat.format(getString("ERROR.GROUPCREATION_FAILED"),
                                   new Object[] {args[i]}));
          rc = ICliBundle.FAIL;
        }
        catch(TWGPersistentObjectSaveException e)
        {
          //TWGRas.error(TWGRas.ENGINE, "SDKCLI.createStaticGroup " + e);
          out.write(MessageFormat.format(getString("ERROR.GROUPCREATION_FAILED"),
                                   new Object[] {args[i]}));
          rc = ICliBundle.FAIL;
        }
      }
    }
    return rc;
  }

  /**
   * delete a group
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int deleteStaticGroup(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.deleteStaticGroup()\n");

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_GROUP"));
      return ICliBundle.USAGE;
    }

    int exitCode = ICliBundle.OK;
    TWGFilter filter = null;

    for (int i = 0; i < args.length; i++)
    {
      // verify that a group with this name exists
      filter = TWGFilter.findFilterByName(args[i]);
      if (filter == null)
      {
        out.write(MessageFormat.format(getString("ERROR.GROUP_NOT_FOUND"),
                                 new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
      else
      {
        // attempt to delete the group
        try
        {
          if (filter.isConsoleDeletable())
          {
            filter.Destroy();
          }
          else
          {
            out.write(MessageFormat.format(getString("ERROR.GROUP_UNDELETEABLE"),
                                     new Object[] {args[i]}));
            exitCode = ICliBundle.FAIL;
          }
        }
        catch(TWGObjectDestroyException e)
        {
          out.write(MessageFormat.format(getString("ERROR.GROUPDELETE_FAILED"),
                                   new Object[] {args[i]}));
          exitCode = ICliBundle.FAIL;
        }
      }
    }

    return exitCode;
  }


  /**
   * Create a managed system
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int createSystem(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.createSystem()\n");

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    for (int i = 0; i < args.length; i++)
    {
      int foundIdx = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
          break;
        }
      }

      // If we've already found a managed system with the same name,
      // return an error to the user saying the managed system name
      // already exists.  We probably should allow managed systems
      // have the same name if they are not the same type.
      if (foundIdx >= 0)
      {
        out.write(MessageFormat.format(getString("ERROR.DUPLICATE"),
                                       new Object[] {mos[foundIdx].getName()}));
        return ICliBundle.FAIL;
      }
      else
      {
        UserDefinedManagedSystem newObject =
                          new UserDefinedManagedSystem(args[i]);
        newObject.setState(TWGManagedObjectConstants.MOSTATE_UNKNOWN);
      }
    }
    return ICliBundle.OK;
  }

  /**
   * delete a managed system
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int deleteSystem(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.deleteSystem()\n");

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    int exitCode = ICliBundle.OK;

    if (args.length < 1)
    {
      out.write(getString("ERROR.SYNTAX"));
      return ICliBundle.USAGE;
    }

    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    for (int i = 0; i < args.length; i++)
    {
      int foundIdx = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
          break;
        }
      }

      // If we haven't found a managed system with the same name,
      // return an error to the user saying the managed system by
      // that name doesn't exist.  We need to think about whether
      // we should destroy all managed systems with the same name,
      // or just destroy the first one we find... not good.
      if (foundIdx == 0)
      {
        out.write(MessageFormat.format(getString("ERROR.NOT_FOUND"),
                                       new Object[] {args[i]}));
        return ICliBundle.FAIL;
      }
      else
      {
        mos[foundIdx].Destroy();
      }
    }
    return ICliBundle.OK;
  }


  /**
   * Add a managed system to a group.  The arguments are: args[0] - the name of the group.
   * args[1-n] - the names of the managed systems to add to the group
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int addSystemToGroup(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.addSystemToGroup()\n");

    int exitCode = ICliBundle.OK;

    if (args.length == 0)
    {
      // no arguments were specified.  The first argument
      // is the group name.
      out.write(getString("ERROR.NO_GROUP"));
      return ICliBundle.USAGE;
    }

    if (args.length < 2)
    {
      // no managed system names were specified.  All arguments
      // after the group name are treated as managed system names.
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    // verify that a group with this name exists
    TWGFilter filter = TWGFilter.findFilterByName(args[0]);
    if (filter == null)
    {
      out.write(MessageFormat.format(getString("ERROR.GROUP_NOT_FOUND"),
                               new Object[] {args[0]}));
      exitCode = ICliBundle.FAIL;
    }
    else
    {
      // verify that the managed system exists and that it
      // isn't aready a member of the specified group

      int[] moids = TWGManagedObject.getAllManagedObjectIDs();
      TWGManagedObjectSet mos = filter.getManagedObjectSet();
      int[] filterMoids = new int[] {};
      if (filter.isManagedObjectSetValid() && (mos != null))
      {
        TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Filter is valid, managed system set retrieved\n");
        filterMoids = mos.getMoids();
      }

      IntValueSet testMOSet = new IntValueSet();
      TWGManagedObject mo = null;

      // start with the first system name parameter, which is args[1]
      for (int i = 1; i < args.length; i++)
      {
        int foundIdx = -1;

        TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Checking argument " + args[0] +
                         " against " + moids.length + " existing managed systems\n");

        // loop through all of the moids to make sure the managed system
        // specified in the argument actually exists.
        for (int j = 0; j < moids.length; j++)
        {
          // get the managed system from the moid
          mo = TWGManagedObject.getManagedObjectByID(moids[j]);
          TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Checking " + args[i] +
                           " against " + mo.getName() + "\n");
          // get the name of the managed system and compare it
          // to the name specified in the argument.
          if (args[i].toUpperCase().equals( mo.getName().toUpperCase() ))
          {
            TWGRas.debug(TWGRas.SDK, "addSystemToGroup: The specified system " +
                             args[i] + " exists\n");
            // this means the managed system specified in the argument
            // list already exists.  This is good.  Now check to see
            // if it is already a member of the specified group.
            TWGManagedObject filtermo = null;
            for (int k = 0; k < filterMoids.length; k++)
            {
              filtermo = TWGManagedObject.getManagedObjectByID(filterMoids[k]);
              if (mo.getName().toUpperCase().equals( filtermo.getName().toUpperCase() ))
              {
                TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Checking existing system " +
                                 mo.getName() + " against group member " +
                                 filtermo.getName() + "\n");
                // this means that the managed system is already
                // a member of the group
                out.write(MessageFormat.format(getString("ERROR.DUPLICATE"),
                                         new Object[] {filtermo.getName()}));
                exitCode = ICliBundle.FAIL;
                foundIdx = -2;
                break;
              }
            } // end loop through managed system ids already in the filter
            foundIdx = (foundIdx == -1) ? j : foundIdx;
            break;
          }
        } // end loop through all existing managed system ids

        // If we found a managed system in the filter with a name matching
        // one of the names in the argument list, display a message to the
        // user saying the managed system is already in the filter.  What
        // happens when there is more than one managed system with the same
        // name in a group?
        if (foundIdx == -1)
        {
          out.write(MessageFormat.format(getString("ERROR.OBJECT_DOESNT_EXIST"),
                                         new Object[] {args[i]}));
          exitCode = ICliBundle.FAIL;

        }
        else if (foundIdx >= 0)
        {
          testMOSet.InsertValue(TWGPersistentObject.toPersistID(moids[foundIdx]));
        }
      } // end for

      // if we have at least one valid managed system name, update the filter
      if (testMOSet.Length() > 0)
      {
        try
        {
          TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Inserting new group members into " +
                           filter.getName() + "\n");
          TWGFilterParms filterParms = new TWGFilterParms(filter.getName(),
                                                          testMOSet,
                                                          TWGFilterParms.ADD_IDS);
          filter.changeParms(filterParms);
          filter.save();

          // the following is for debug purposes only
          TWGRas.debug(TWGRas.SDK, "addSystemToGroup: The members of the group " +
                           filter.getName() + " are:\n");
          TWGManagedObjectSet filtermos = filter.getManagedObjectSet();

          if (filtermos != null)
          {
            int[] newfiltermoids = filtermos.getMoids();
            for (int l = 0; l < newfiltermoids.length; l++)
            {
              // get the managed system from the moid
              TWGManagedObject filtermo = TWGManagedObject.getManagedObjectByID(newfiltermoids[l]);
              TWGRas.debug(TWGRas.SDK, "addSystemToGroup: " + filtermo.getName() + "\n");
            }
          }
          else
          {
            TWGRas.debug(TWGRas.SDK, "addSystemToGroup: The managed system set is null\n");
          }
        }
        catch(TWGFilterException e)
        {
          out.write(MessageFormat.format(getString("ERROR.ALLADDTOGROUP_FAILED"),
                                         new Object[] {filter.getName()}));
          TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Exception = " + e.toString() + "\n");
          exitCode = ICliBundle.FAIL;
        }
        catch(TWGPersistentObjectSaveException e)
        {
          out.write(MessageFormat.format(getString("ERROR.ALLADDTOGROUP_FAILED"),
                                         new Object[] {filter.getName()}));
          TWGRas.debug(TWGRas.SDK, "addSystemToGroup: Exception = " + e.toString() + "\n");
          exitCode = ICliBundle.FAIL;
        }
      } // end if
    }
    return exitCode;
  }

  /**
   * Delete a managed system from a group.  The arguments are: args[0] - the name of the group.
   * args[1-n] - the names of the managed systems to delete from the group
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int removeSystemFromGroup(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "removeSystemFromGroup: In SDKCLI.removeSystemFromGroup()\n");

    int exitCode = ICliBundle.OK;

    if (args.length == 0)
    {
      // no arguments were specified.  The first argument
      // is the group name.
      out.write(getString("ERROR.NO_GROUP"));
      return ICliBundle.USAGE;
    }

    if (args.length < 2)
    {
      // no managed system names were specified.  All arguments
      // after the group name are treated as managed system names.
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    // verify that a group with this name exists
    TWGFilter filter = TWGFilter.findFilterByName(args[0]);
    if (filter == null)
    {
      out.write(MessageFormat.format(getString("ERROR.GROUP_NOT_FOUND"),
                               new Object[] {args[0]}));
      exitCode = ICliBundle.FAIL;
    }
    else
    {
      // verify that the managed system exists and that
      // it is a member of the specified group
      IntValueSet testMOSet = null;
      int[] moids = null;

      TWGManagedObjectSet mos = filter.getManagedObjectSet();
      if (mos != null)
      {
        moids = mos.getMoids();
        testMOSet = new IntValueSet(moids, 0, moids.length);
      }
      else
      {
        // the filter isn't active, try another way
        moids = TWGManagedObject.getAllManagedObjectIDs();
        IntValueSet fullMOSet = new IntValueSet(moids, 0, moids.length);
        testMOSet = new IntValueSet();
        filter.processNewChangedManagedObjects(fullMOSet, testMOSet);
      }

      TWGManagedObject mo = null;

      // start with the first system name parameter, which is args[1]
      for (int i = 1; i < args.length; i++)
      {
        int foundIdx = -1;
        // loop through all of the moids to make sure the managed system
        // specified in the argument actually exists.
        for (int j = 0; j < moids.length; j++)
        {
          // get the managed system from the moid
          mo = TWGManagedObject.getManagedObjectByID(moids[j]);
          TWGRas.debug(TWGRas.SDK, "removeSystemFromGroup: Checking " + args[i] +
                       " against " + mo.getName() + "\n");
          // get the name of the managed system and compare it
          // to the name specified in the argument.
          if (args[i].toUpperCase().equals( mo.getName().toUpperCase() ))
          {
            TWGRas.debug(TWGRas.SDK, "removeSystemFromGroup: " + mo.getName() +
                         " is in the group " + filter.getName() + "\n");
            // this means the managed system specified in the argument
            // list already exists in the group.  This is good.
            foundIdx = j;
            break;
          }
        }

        // The purpose of the following code section is to print out a more
        // descriptive error message to the user other than a generic "request
        // failed" message.
        // If we didn't find a managed system in the filter with a name matching
        // one of the names in the argument list, find out if the managed system
        // even exists.  Then display a message to the user saying the managed
        // system doesn't exist in the filter or doesn't exist at all.  What
        // happens when there is more than one managed system with the same
        // name in a group?
        if (foundIdx == -1)
        {
          int[] allmoids = TWGManagedObject.getAllManagedObjectIDs();
          TWGRas.debug(TWGRas.SDK, "removeSystemFromGroup: " + args[i] +
                       "is NOT in the group " + filter.getName() + "\n");

          for (int k = 0; k < allmoids.length; k++)
          {
            // get the managed system from the moid
            mo = TWGManagedObject.getManagedObjectByID(allmoids[k]);
            TWGRas.debug(TWGRas.SDK, "removeSystemFromGroup: Checking " +
                         args[i] + " against " + mo.getName() + "\n");
            // get the name of the managed system and compare it
            // to the name specified in the argument.
            if (args[i].toUpperCase().equals( mo.getName().toUpperCase() ))
            {
              // the system exists but isn't a member of the group
              out.write(MessageFormat.format(getString("ERROR.OBJECT_NOT_A_MEMBER"),
                                         new Object[] {mo.getName(), filter.getName()}));
              foundIdx = -2;
              break;
            }
          }
          if (foundIdx == -1)
          {
            // the requested system doesn't exist
            out.write(MessageFormat.format(getString("ERROR.OBJECT_DOESNT_EXIST"),
                                         new Object[] {args[i]}));
          }
          exitCode = ICliBundle.FAIL;
        }
        else
        {
          testMOSet.RemoveValue(TWGPersistentObject.toPersistID(moids[foundIdx]));
        }
      } // end for

      // if we have at least one valid managed system name, update the filter
      //if (testMOSet.Length() > 0)
      //{
        try
        {
          TWGFilterParms filterParms = new TWGFilterParms(filter.getName(),
                                                          testMOSet,
                                                          TWGFilterParms.REPLACE_IDS);
          filter.changeParms(filterParms);
          filter.save();
        }
        catch(TWGFilterException e)
        {
          out.write(MessageFormat.format(getString("ERROR.REMOVEFROMGROUP_FAILED"),
                                         new Object[] {filter.getName(), mo.getName(), e}));
          exitCode = ICliBundle.FAIL;
        }
        catch(TWGPersistentObjectSaveException e)
        {
          out.write(MessageFormat.format(getString("ERROR.REMOVEFROMGROUP_FAILED"),
                                         new Object[] {filter.getName(), mo.getName(), e}));
          exitCode = ICliBundle.FAIL;
        }
      //}
    }
    return exitCode;


  }


  /**
   * List all systems in a group.  The arguments are: args[0] - the name of the group.
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of group names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int listSystemsInGroup(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.listSystemsInGroup()\n");

    int exitCode = ICliBundle.OK;

    if (args.length == 0)
    {
      // no arguments were specified.  The first argument
      // is the group name.
      out.write(getString("ERROR.NO_GROUP"));
      return ICliBundle.USAGE;
    }

    // start with the first group name parameter, which is args[0]
    for (int i = 0; i < args.length; i++)
    {
      // verify that a group with this name exists
      TWGFilter filter = TWGFilter.findFilterByName(args[i]);
      if (filter == null)
      {
        out.write(MessageFormat.format(getString("ERROR.GROUP_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
      else
      {
        // get the list of managed systems for that group

        IntValueSet testMOSet = null;
        int[] moids = null;

        TWGManagedObjectSet mos = filter.getManagedObjectSet();
        if (mos != null)
        {
          moids = mos.getMoids();
          testMOSet = new IntValueSet(moids, 0, moids.length);
        }
        else
        {
          // the filter isn't active, try another way
          moids = TWGManagedObject.getAllManagedObjectIDs();
          IntValueSet fullMOSet = new IntValueSet(moids, 0, moids.length);
          testMOSet = new IntValueSet();
          filter.processNewChangedManagedObjects(fullMOSet, testMOSet);
        }

        TWGManagedObject mo = null;

        int foundIdx = -1;
        // loop through all of the moids to make sure the managed system
        // specified in the argument actually exists.
        if (moids.length <= 0)
        {
          out.write(MessageFormat.format(getString("INFO.GROUP_HAS_NO_MEMBERS"),
                                         new Object[] {args[i]}));
        }
        else
        {
          out.write(MessageFormat.format(getString("INFO.GROUP_MEMBERS"),
                                         new Object[] {args[i]}));
          for (int j = 0; j < testMOSet.Length(); j++)
          {
            // get the managed system from the moid
            mo = TWGManagedObject.getManagedObjectByID(testMOSet.GetValue(j));
            out.write(mo.getName() + "\n");
          }
          out.write("\n");
        }
      }
    }
    return exitCode;
  }


  /**
   * Add an attribute to a user defined managed system.  The arguments are:
   * args[0] - attribute name
   * args[1] - description
   * args[2] - value
   * args[3-n] - system names
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int addAttribute(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.addAttribute()\n");
    int exitCode = ICliBundle.OK;

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_NAME"));
      return ICliBundle.USAGE;
    }

    else if (args.length == 1)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_DESCRIPTION"));
      return ICliBundle.USAGE;
    }

    else if (args.length == 2)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_VALUE"));
      return ICliBundle.USAGE;
    }

    else if (args.length == 3)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    // Verify that the systems specified exist
    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the fourth argument,
    // which should be our first system name
    for (int i = 3; i < args.length; i++)
    {
      int foundIdx = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
          break;
        }
      }

      // Display agents if system found
      if (foundIdx >= 0)
      {
        if (!(mos[foundIdx] instanceof UserDefinedManagedSystem))
        {
          // the system exists but since it's not a UserDefinedManagedSystem,
          // we can't update the attributes.
          out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_MANAGED_SYSTEM"),
                                                   new Object[] {mos[foundIdx].getName()}));
          exitCode = ICliBundle.FAIL;
        }
        else
        {
          UserDefinedManagedSystem userObject = (UserDefinedManagedSystem) mos[foundIdx];
          // add the attribute to the system
          if (userObject.addAttribute(args[0], args[1], new TWGString(args[2]), locale) != true)
          {
            // this means that the attribute has already been defined for that system
            // LJI - start here
            out.write(MessageFormat.format(getString("ERROR.DUPLICATE_ATTRIBUTE"),
                                                     new Object[] {args[0]}));
            exitCode = ICliBundle.FAIL;
          }
          else
          {
            // the attribute was added successfully.

          }
        }
      }
      else
      {
        // the managed system couldn't be found.  I need to rename the message.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    }
    return exitCode;
  }


  /**
   * Remove an attribute from a user defined managed system.
   * The arguments are:  args[0] - attribute name,
   * args[1-n] - the name of the user defined managed system
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int removeAttribute(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.removeAttribute()\n");
    int exitCode = ICliBundle.OK;


    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_NAME"));
      return ICliBundle.USAGE;
    }
    else if (args.length == 1)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }


    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the second argument,
    // which should be our first system name
    for (int i = 1; i < args.length; i++)
    {
      int foundIdx = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
          break;
        }
      }

      // Display agents if system found
      if (foundIdx >= 0)
      {
        out.write(mos[foundIdx].getName() + "\n");
        if (!(mos[foundIdx] instanceof UserDefinedManagedSystem))
        {
          // the system exists but since it's not a UserDefinedManagedSystem,
          // we can't update the attributes.
          out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_MANAGED_SYSTEM"),
                                                   new Object[] {mos[foundIdx].getName()}));
          exitCode = ICliBundle.FAIL;
        }
        else
        {
          UserDefinedManagedSystem userObject = (UserDefinedManagedSystem) mos[foundIdx];
          // verify that the system contains the attribute we're looking for
          if (userObject.removeAttribute(args[0], locale) == false)
          {
            // this means that the attribute has already been defined for that system
            out.write(MessageFormat.format(getString("ERROR.ATTRIBUTE_DOESNT_EXIST"),
                                                     new Object[] {args[0]}));
            exitCode = ICliBundle.FAIL;
          }
          else
          {
            // the attribute was removed successfully
          }
        }
      }
      else
      {
        // the managed system couldn't be found.  I need to rename the message.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    }
    return ICliBundle.OK;
  }


  /**
   * List the attributes of a managed system.
   * The arguments are:  args[0-n] - the names of the managed systems
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int listAttributes(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.listAttributes()\n");
    int exitCode = ICliBundle.OK;


    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }


    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the second argument,
    // which should be our first system name
    for (int i = 0; i < args.length; i++)
    {
      int numberFound = 0;
      for (int j = 0; j < mos.length; j++)
      {
        int foundIdx = -1;
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
          numberFound++;
        }

        // Display attributes if system found
        // Since it is possible to have more than one system with the same
        // name, loop through and display attributes for all of the systems
        // with the same name.
        if (foundIdx >= 0)
        {
          //out.write(mos[foundIdx].getName() + "\n");

          TWGManagedObject userObject = mos[foundIdx];
          // get the list of attribute ids for the object
          String[] attribIds = userObject.getAttributeIDList();

          // verify that there are attribute ids
          if (attribIds.length == 0)
          {
            // this means that the system doesn't have any attribute ids
            out.write(MessageFormat.format(getString("INFO.SYSTEM_HAS_NO_ATTRIBUTES"),
                                                     new Object[] {args[i]}));
          }
          else
          {
            // if this is the first system with a matching name that we've
            // found, print out a generic attribute message
            if (numberFound == 1)
            {
              out.write(MessageFormat.format(getString("INFO.ATTRIBUTE_LIST"),
                                                       new Object[] {mos[foundIdx].getName()}));
            }
            // if this is an additional system with a matching name, print
            // out a message indicating that this is another system we've found
            else
            {
              out.write(MessageFormat.format(getString("INFO.ANOTHER_ATTRIBUTE_LIST"),
                                                       new Object[] {mos[foundIdx].getName()}));
            }
            // loop through the attribute ids for the managed system, get the
            // description and value of the attribute, and print them out.
            String attrDesc = null;
            DataValue attrValue = null;

            for (int k = 0; k < attribIds.length; k++)
            {
              attrDesc = userObject.getAttributeDescription(attribIds[k], locale);

              // if we're looking for the factory name, give a string
              // description instead of the factory id.
              if (attrDesc.equals(engine_mlb.getString(locale, "attrib.lbl.factor_id", "attrib.lbl.factor_id")))
              {
                attrValue = new TWGString(userObject.getMof().getName());
              }
              // if we're looking for the status, give a string
              // description instead of the statis id.
              else if (attrDesc.equals(engine_mlb.getString(locale,
                                                            "attrib.lbl.state",
                                                            "attrib.lbl.state")))
              {
                int state = userObject.getState();
                if ((state >= 0) && (state < state_ids.length))
                {
                  attrValue = new TWGString(engine_mlb.getString(locale,
                                                                 state_ids[state],
                                                                 state_ids[state]));
                }
                else
                {
                  attrValue = new TWGString(getString("STATUS.UNKNOWN"));
                }
              }
              // we should be ok for the rest
              else
              {
                attrValue = userObject.getAttributeValue(attribIds[k], locale);
              }
              out.write(attrDesc + ":  " + attrValue + "\n");
            }
            out.write("\n");
          }
        }
      } // end for loop through managed object list

      if (numberFound == 0)
      {
        // the managed system couldn't be found.  I need to rename the message.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    } // end for loop through arg list

    return ICliBundle.OK;
  }


  /**
   * List the attributes of a managed system.
   * The arguments are:  args[0-n] - the names of the managed systems
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int listUserDefinedAttributes(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.listUserDefinedAttributes()\n");
    int exitCode = ICliBundle.OK;


    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_NAME"));
      return ICliBundle.USAGE;
    }
    else if (args.length == 1)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }


    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the second argument,
    // which should be our first system name
    for (int i = 0; i < args.length; i++)
    {
      int numberFound = 0;
      for (int j = 0; j < mos.length; j++)
      {
        int foundIdx = -1;
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
        }

        // Display attributes if system found
        // Since it is possible to have more than one system with the same
        // name, loop through and display attributes for all of the systems
        // with the same name.
        if (foundIdx >= 0)
        {
          numberFound++;
          //out.write(mos[foundIdx].getName() + "\n");
          if (!(mos[foundIdx] instanceof UserDefinedManagedSystem))
          {
            // the system exists but since it's not a UserDefinedManagedSystem,
            // we can't update the attributes.
            out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_MANAGED_SYSTEM"),
                                                     new Object[] {mos[foundIdx].getName()}));
            exitCode = ICliBundle.FAIL;

          }
          else
          {
            UserDefinedManagedSystem userObject = (UserDefinedManagedSystem) mos[foundIdx];
            // get the list of attribute ids for the object
            String[] attribIds = userObject.getUserDefinedAttributeIDList();

            // verify that there are attribute ids
            if (attribIds.length == 0)
            {
              // this means that the system doesn't have any attribute ids
              out.write(MessageFormat.format(getString("INFO.SYSTEM_HAS_NO_USER_DEFINED_ATTRIBUTES"),
                                                       new Object[] {args[i]}));
            }
            else
            {
              out.write(MessageFormat.format(getString("INFO.USER_DEFINED_ATTRIBUTE_LIST"),
                                                       new Object[] {mos[foundIdx].getName()}));
              for (int k = 0; k < attribIds.length; k++)
              {
                out.write(attribIds[k] + "\n");
              }
            } // end if attribIds.length > 0
          } // end if instanceof UserDefinedManagedSystem

          // since we've found a user defined managed object and since no two
          // two user defined managed objects can have the same name, we're
          // done with this name so break out of loop
          break;

        } // end if foundIdx >= 0
      } // end for loop through managed object list

      if (numberFound == 0)
      {
        // the managed system couldn't be found.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    } // end for loop through arg list

    return ICliBundle.OK;
  }


  /**
   * List user defined systems with the specified user defined attribute and value
   * The arguments are: args[0] - the attribute id=value pair
   *                    args[1-n] - the names of the managed systems
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int listSystemByUserDefinedAttribute(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.listUserDefinedSystemByAttribute()\n");
    int exitCode = ICliBundle.OK;


    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_ATTRIBUTE_NAME"));
      return ICliBundle.USAGE;
    }
    else if (args.length == 1)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }


    String attrib = null;
    String value = null;
    int equals = args[0].indexOf('=');
    if (equals >= 1)
    {
      attrib = args[0].substring(0, equals);
      value  = args[0].substring(equals+1);
    }
    else
    {
      out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_ATTRIBUTE"),
                                               new Object[] {args[0]}));
      return ICliBundle.FAIL;
    }



    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the second argument,
    // which should be our first system name
    for (int i = 1; i < args.length; i++)
    {
      int numberFound = 0;
      for (int j = 0; j < mos.length; j++)
      {
        int foundIdx = -1;
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
        }

        // Display attributes if system found
        // Since it is not possible to have more than one user defined system
        // with the same, stop looping after we find a matching system
        // name, loop through and display attributes for all of the systems
        if (foundIdx >= 0)
        {
          //out.write(mos[foundIdx].getName() + "\n");
          if (!(mos[foundIdx] instanceof UserDefinedManagedSystem))
          {
            // the system exists but since it's not a UserDefinedManagedSystem,
            // we can't update the attributes.
            out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_MANAGED_SYSTEM"),
                                                     new Object[] {mos[foundIdx].getName()}));
            exitCode = ICliBundle.FAIL;
          }
          else
          {
            numberFound++;
            UserDefinedManagedSystem userObject = (UserDefinedManagedSystem) mos[foundIdx];

            // get the list of attribute ids for the object
            String[] attribIds = userObject.getUserDefinedAttributeIDList();

            // verify that there are attribute ids
            if (attribIds.length == 0)
            {
              // this means that the system doesn't have any attribute ids
              out.write(MessageFormat.format(getString("INFO.SYSTEM_HAS_NO_USER_DEFINED_ATTRIBUTES"),
                                                       new Object[] {args[i]}));
            }
            else
            {
              boolean foundAttr = false;
              for (int k = 0; k < attribIds.length; k++)
              {
                if (attrib.toUpperCase().equals(attribIds[k].toUpperCase()))
                {
                  // if this is the first system with a matching name that we've
                  // found, print out a generic attribute message
                  if (numberFound == 1)
                  {
                    out.write(MessageFormat.format(getString("INFO.USER_DEFINED_SYSTEMS_WITH_ATTRIBUTE_LIST"),
                                                         new Object[] {attribIds[k]}));
                  }

                  out.write(userObject.getName() + "\n");
                  foundAttr = true;
                  // we're only looking to match one attribute and we've
                  // found it, so break out of the loop
                  break;
                }
              } // end loop through attributes

              if (foundAttr == false)
              {
                out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_ATTRIBUTE"),
                                               new Object[] {attrib}));
              }
            } // end if attribIds.length > 0
          } // end if instanceof UserDefinedManagedSystem

          // since we've found and handled the only user defined managed
          // system with this name, break out of the loop through all
          // of the user defined managed systems
          break;

        } // end if foundIdx >= 0
      } // end for loop through managed object list

      if (numberFound == 0)
      {
        // the managed system couldn't be found.  I need to rename the message.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    } // end for loop through arg list

    return ICliBundle.OK;
  }


  /**
   * Change a user defined attribute in a user defined managed system
   * The arguments are: args[0] - the attribute id=value pair
   *                    args[1-n] - the names of the managed systems
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int changeUserDefinedAttributeValue(Locale locale, String[] args, Writer out) throws Exception
  {
    // note - this method is very similar to the previous method.  Might want to
    // do some work in the future to combine some tasks.

    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.changeUserDefinedAttributeValue()\n");
    int exitCode = ICliBundle.OK;


    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    String attrib = null;
    String value = null;
    int equals = args[0].indexOf('=');
    if (equals >= 1)
    {
      attrib = args[0].substring(0, equals);
      value  = args[0].substring(equals+1);
    }
    else
    {
      out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_ATTRIBUTE"),
                                               new Object[] {args[0]}));
      return ICliBundle.FAIL;
    }



    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    // start processing arguments at the second argument,
    // which should be our first system name
    for (int i = 1; i < args.length; i++)
    {
      int numberFound = 0;
      for (int j = 0; j < mos.length; j++)
      {
        int foundIdx = -1;
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundIdx = j;
        }

        // Display attributes if system found
        // Since it is not possible to have more than one user defined system
        // with the same, stop looping after we find a matching system
        // name, loop through and display attributes for all of the systems
        if (foundIdx >= 0)
        {
          //out.write(mos[foundIdx].getName() + "\n");
          if (!(mos[foundIdx] instanceof UserDefinedManagedSystem))
          {
            // the system exists but since it's not a UserDefinedManagedSystem,
            // we can't update the attributes.
            out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_MANAGED_SYSTEM"),
                                                     new Object[] {mos[foundIdx].getName()}));
            exitCode = ICliBundle.FAIL;
          }
          else
          {
            numberFound++;
            UserDefinedManagedSystem userObject = (UserDefinedManagedSystem) mos[foundIdx];

            // get the list of attribute ids for the object
            String[] attribIds = userObject.getUserDefinedAttributeIDList();

            // verify that there are attribute ids
            if (attribIds.length == 0)
            {
              // this means that the system doesn't have any attribute ids
              out.write(MessageFormat.format(getString("INFO.SYSTEM_HAS_NO_USER_DEFINED_ATTRIBUTES"),
                                                       new Object[] {args[i]}));
            }
            else
            {
              boolean foundAttr = false;
              for (int k = 0; k < attribIds.length; k++)
              {
                if (attrib.toUpperCase().equals(attribIds[k].toUpperCase()))
                {
                  // if this is the first system with a matching name that we've
                  // found, print out a generic attribute message
                  userObject.setAttributeValue(attribIds[k], new TWGString(value));
                  out.write(MessageFormat.format(getString("INFO.ATTRIBUTE_UPDATE_SUCCESSFUL"),
                                                           new Object[] {attribIds[k], value,
                                                                         userObject.getName()}));
                  foundAttr = true;
                  // we're only looking to match one attribute and we've
                  // found it, so break out of the loop
                  break;
                }
              } // end loop through attributes

              if (foundAttr == false)
              {
                out.write(MessageFormat.format(getString("ERROR.NOT_A_USER_DEFINED_ATTRIBUTE"),
                                               new Object[] {attrib}));
              }
            } // end if attribIds.length > 0
          } // end if instanceof UserDefinedManagedSystem

          // since we've found and handled the only user defined managed
          // system with this name, break out of the loop through all
          // of the user defined managed systems
          break;

        } // end if foundIdx >= 0
      } // end for loop through managed object list

      if (numberFound == 0)
      {
        // the managed system couldn't be found.  I need to rename the message.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
        exitCode = ICliBundle.FAIL;
      }
    } // end for loop through arg list

    return ICliBundle.OK;
  }


  /**
   * Get the status of a managed system
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int getStatus(Locale locale, String[] args, Writer out) throws Exception
  {
    TWGRas.entryExit(TWGRas.SDK, "In SDKCLI.getStatus()\n");

    if (args.length == 0)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();
    for (int i = 0; i < args.length; i++)
    {
      int foundObjectIndex = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundObjectIndex = j;
          break;
        }
      }

      // Display agents if system found
      if (foundObjectIndex >= 0)
      {
        out.write(mos[foundObjectIndex].getName() + ":  ");
        int status = mos[foundObjectIndex].getState();
        switch (status)
        {
          case TWGManagedObject.MOSTATE_ERROR_OFFLINE:
          case TWGManagedObject.MOSTATE_NORMAL_OFFLINE:
            out.write(getString("STATUS.OFFLINE"));
            break;
          case TWGManagedObject.MOSTATE_ERROR_ONLINE:
          case TWGManagedObject.MOSTATE_NORMAL_ONLINE:
            out.write(getString("STATUS.ONLINE"));
            break;
          case TWGManagedObject.MOSTATE_INVALID_OBJECT:
          case TWGManagedObject.MOSTATE_INDETERMINATE:
          case TWGManagedObject.MOSTATE_UNKNOWN:
          case TWGManagedObject.MOSTATE_UNLICENSED:
          default:
            out.write(getString("STATUS.UNKNOWN"));
            break;
        }
      }
      else
      {
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
      }
    }
    return ICliBundle.OK;
  }

  /**
   * set the status for a managed system
   * the arguments expected are arg[0] - status, arg[1-n] - system name
   * @param Locale locale - client locale for NLS output
   * @param String[] args - the command arguments (contains a list of system names)
   * @param Writer out - output stream back to client
   * @return int - client application exit code
   */
  public int setStatus(Locale locale, String[] args, Writer out) throws Exception
  {
    int status = TWGManagedObject.MOSTATE_UNKNOWN;


    // check parameters
    if (args.length == 0)
    {
      out.write(getString("STATUS.NOT_SPECIFIED"));
      return ICliBundle.USAGE;
    }

    if (args.length == 1)
    {
      out.write(getString("ERROR.NO_SYSTEM"));
      return ICliBundle.USAGE;
    }

    if (args[0].toUpperCase().equals(getString("STATUS.ONLINE").toUpperCase()))
    {
      status = TWGManagedObject.MOSTATE_NORMAL_ONLINE;
    }
    else if (args[0].toUpperCase().equals(getString("STATUS.OFFLINE").toUpperCase()))
    {
      status = TWGManagedObject.MOSTATE_NORMAL_OFFLINE;
    }
    else if (args[0].toUpperCase().equals(getString("STATUS.UNKNOWN").toUpperCase()))
    {
      status = TWGManagedObject.MOSTATE_UNKNOWN;
    }
    else
    {
      out.write(MessageFormat.format(getString("STATUS.UNDEFINED"),
                                     new Object[] {args[0]}));
      return ICliBundle.USAGE;
    }


    // if we made it to here, the parameters are ok and it's time
    // to start processing the command
    TWGManagedObject[] mos = TWGManagedObject.getAllManagedObjects();

    // start at the first system name
    for (int i = 1; i < args.length; i++)
    {
      int foundObjectIndex = -1;
      for (int j = 0; j < mos.length; j++)
      {
        if (args[i].toUpperCase().equals( mos[j].getName().toUpperCase() ))
        {
          foundObjectIndex = j;
          break;
        }
      }

      // Display agents if system found
      if ((foundObjectIndex >= 0) && (args[0] != null))
      {
          mos[foundObjectIndex].setState(status);
          out.write(MessageFormat.format(getString("STATUS.UPDATE_SUCCESSFUL"),
                                         new Object[] {mos[foundObjectIndex].getName()}));
      }
      else
      {
        // The managed system doesn't exist.
        out.write(MessageFormat.format(getString("ERROR.STATUS_NOT_FOUND"),
                                       new Object[] {args[i]}));
      }
    }
    return ICliBundle.OK;
  }

  public String getString(String id)
  {
    String name = id;
    if (mlb != null)
    {
      name = mlb.getString(Locale.getDefault(), id, id);
    }
    return name;
  }
}
