/* Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2003 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

package com.ibm.sysmgt.sdk.switchmgt;



import com.ibm.sysmgt.switchmgt.SMAppLauncher;
import com.ibm.sysmgt.switchmgt.SwitchIdentifier;

import com.tivoli.twg.engine.TWGSubtask;
import com.tivoli.twg.engine.TWGConTask;
import com.tivoli.twg.engine.TWGTaskActivator;
import com.tivoli.twg.console.TWGMainGUI;

import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.log.TWGRas;
/**
 * SDK Utility<BR>
 * Director Task Launcher PopIn for use with the Switch Management LaunchPad.
 * This general purpose Director task launcher implementation example takes the
 * specific task and subtask pair as an input.<P>
 *
 * This class would be identified in a switch definition property file as the
 * interface or gateway into activating a management task.  The format of the
 * management definition would be defined as below:<BR>
 * com.ibm.sysmgt.sdk.switchmgt.SMTaskLauncher TaskAndSubTaskDefinition<BR>
 * <BR>Where the TaskAndSubTaskDefinition is defined as such:
 * <BR>your Director task ID value                  --> TaskID
 * <BR>Then a separator                             --> @@
 * <BR>Then your Director tasks subtask to execute  --> SubTaskID
 * <BR>Together the form is --> "MyTaskID@@MySubTaskID"
 * <BR>A TaskAndSubTaskDefinition example is --> Tivoli|RemoteSession@@RemoteSessionID
 * <BR>Below is the corresponding full definition value:
 * <BR>com.ibm.sysmgt.sdk.switchmgt.SMTaskLauncher Tivoli|RemoteSession@@RemoteSessionID
 *
 * <P>Note: This "General Purpose" Switch Management LaunchPad Director task launcher is not the
 * means of doing a <CODE>com.ibm.sysmgt.switchmgt.SMTaskLauncher</CODE> implementation do
 * to problems with the providing the displayable NLS product name.  And hence it is a better
 * design to provide a <CODE>com.ibm.sysmgt.switchmgt.SMTaskLauncher</CODE> implementation per
 * specific switch type.
 *
 * @author Rob Warren
 * @version 1.0
 */
public class SMTaskLauncher implements SMAppLauncher {

    /**
     * Seperator value used by mgmt application definitions
     * to breakup the task and subtask values
     */
    private static final String TASK_SEPERATOR = "@@";

    /**
     * Invoked by the SMLauncher to execute a management application against a specific target
     * @param module The switch module target to be managed
     * @param args   This should be an array with 1 value defining a directorTask\subTask combo
     *               of the form: MyTaskID@@MySubTaskID
     * @return <CODE>true</CODE> for success, <CODE>false</CODE> for failure
     */
    public boolean runApplication(SwitchIdentifier module, String[] args) {

        boolean launchStatus = true;

        // Debug
        System.out.println("SMTaskLauncher.runApplication() enter.");
        System.out.println("SMTaskLauncher.runApplication() args = " + args);
        // Debug

        if((args == null) || (args.length < 1)) {
            System.out.println("SMTaskLauncher.runApplication() ERROR <null> args");
            return false;
        }

        // Debug
        System.out.println("SMTaskLauncher.runApplication() args.length = " + args.length);
        System.out.println("SMTaskLauncher.runApplication() args[0] = " + args[0]);
        // Debug

        if(module == null) {
            System.out.println("SMTaskLauncher.runApplication() ERROR <null> module");
            return false;
        }
        if(module.getMoid() <= 0) {
            TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.runApplication() Invalid moid " + module.getMoid());
            return false;
        }

        try {
            TWGSubtask subtask = SMTaskLauncher.getMgmtDefDirectorSubtask(args[0]);
            if(subtask == null) {
                TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.runApplication() Error loading subtask " + args[0]);
                return false;
            }

            LongValueSet moidSet = new LongValueSet(1);
            moidSet.InsertValue(module.getMoid());
            new TWGTaskActivator(subtask, moidSet, null, TWGMainGUI.getMainConsole(), null, null);
        }
        catch (Exception e) {
            TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.runApplication() " + e);
            launchStatus = false;
        }

        return launchStatus;
    }

    /**
     * Return a name of implemented management application which can be displayed to a user
     * <BR>Note that this implementation is not NLS compliant.  Language resource bundles are
     * needed for actual implementations.
     * @return NLS string of the switch type product name
     */
    public String getNLSName() {
        return "Switch Management Director Task Launcher";
    }

    /**
     * Get Director task specified in mgmt definition
     * @param  SMDef Director task definition string
     *         of the format "DirectorTask:MyTaskID@@MySubTaskID"
     * @return Console subtask object specified in definition,
     *         else <CODE>null</CODE> for loading error
     */
    static TWGSubtask getMgmtDefDirectorSubtask(String mgmtAppDef) {

        int mark = mgmtAppDef.indexOf(SMTaskLauncher.TASK_SEPERATOR);
        if(mark < 0) {
            TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.getMgmtDefDirectorSubtask() InvalidTaskDefStr " + mgmtAppDef);
            return null;
        }

        String taskID = mgmtAppDef.substring(0, mark);
        String subtaskID = mgmtAppDef.substring(mark+SMTaskLauncher.TASK_SEPERATOR.length(), mgmtAppDef.length());

        // Debug
        System.out.println("SMTaskLauncher.getMgmtDefDirectorSubtask() mgmtAppDef " + mgmtAppDef);
        System.out.println("SMTaskLauncher.getMgmtDefDirectorSubtask() task = " + taskID);
        System.out.println("SMTaskLauncher.getMgmtDefDirectorSubtask() subTask = " + subtaskID);
        // Debug

        if((taskID == null) || (subtaskID == null)) {
            TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.getMgmtDefDirectorSubtask() taskID or subtask ID null ");
            return null;
        }

        TWGConTask task = TWGConTask.getTask(taskID);
        if(task == null) {
            TWGRas.error(TWGRas.CHASSIS, "SMTaskLauncher.getMgmtDefDirectorSubtask() Could not find task " + taskID);
            return null;
        }
        return task.getSubtask(subtaskID);
    }
}
