/***************************************************************************
 * IBM Confidential                                             *
 *                                                                         *
 * OCO Source Materials                                                    *
 *                                                                         *
 * program number (04L8267)                                                *
 *                                                                         *
 * (C) Copyright 1998 by IBM                    *
 *                                                                         *
 * The source code for this program is not published or otherwise divested *
 * of its trade secrets, irrespective of what has been deposited with the  *
 * U.S. Copyright Office.                                                  *
 ***************************************************************************/
package com.BobCo.datalinks;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import com.tivoli.twg.libs.*;
import java.util.Locale;

/**
 * Sample Director TCP/IP-base <code>TWGDataLink</code> implementation.  
 * This class implements a sample TCP/IP based communication driver for
 * supporting communications between the Director Console and Server.
 * The sample provides all the required methods, defined in the 
 * <code>TWGDataLink</code> interface, to support server-based and console-based
 * use of this class for unencrypted TCP/IP-based communications.
 * <p>The class implementation supports an optional initialization string, which
 * currently only supports an override value for the IP port number used by the
 * server-based implmentation.  If provided, the string can consist of a number
 * representing the decimal value for the server's port address (default is 2034).
 * <p>Any modification of this sample should use a different port address default.
 * 
 * @see com.tivoli.twg.libs.TWGDataLink
 *
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class TCPIPDataLink implements TWGDataLink {
    /**
     * Default port number for server socket (set to 2034)
     */
    protected static final int default_portnum = 2034;
    protected Socket    sock;
    protected ServerSocket server_sock;
    protected boolean   is_server_sock;
    protected int       portnum;
    protected boolean   is_inited;
    /**
     * Default constructor for TCPIPDataLink.  Initializes attributes.
     */
    public      TCPIPDataLink() {
        is_inited = false;
    }
    /**
     * Internal constructor used by server <code>TCPIPDataLink</code> to build 
     * instances when sessions are accepted.
     * @param s - socket to use for session
     */
    protected   TCPIPDataLink(Socket s) {
        is_inited = true;
        sock = s;
        is_server_sock = false;
    }
    /**
     * Initialize a console <code>TCPIPDataLink</code> instance.  Passes in a 
     * parameter string which can either be null or include a decimal number providing
     * an override to the default port number (or * for no override).  
     * Called after constructor and before <code>TWGOpenLink()</code> is called.  
     * @param init_str - initialization string from link configuration
     * @exception  <code>IOException</code> if communications error
     * @exception  <code>TWGInitParmException</code> if bad initialization string
     * @exception  <code>TWGLinkNotSupportedException</code> if link type not supported
     */
    public void TWGInitializeConsoleLink(String init_str) throws IOException,
        TWGInitParmException, TWGLinkNotSupportedException {
        portnum = default_portnum;
        /* If we've got an initialization string, parse it		    */
        if(init_str != null) {
            String s = init_str.trim();
            if((s.length() > 0) && (!s.startsWith("*"))) {
                try {
                    portnum = Integer.parseInt(s);
                } catch(NumberFormatException n) {
                    throw new TWGInitParmException(
                        "TWGInitializeConsoleLink: Bad Initialization string - " + 
                        init_str);
                }
            }
        }
        is_inited = true;	/* Mark as initialized			    */
        is_server_sock = false; /* Is setup as console socket               */
    }
    /**
     * Open connection to server, using given host/target name.  User-id
     * and password are not used for connecting.  
     *
     * @param   hostname - string for identifying target server
     * @param   userid - string for userid to attempt for logon
     * @param   password - string for password for logon
     * @return  true if connection is successful, or false if not
     * @exception       <code>UnknownHostException</code> if bad hostname
     * @exception       <code>IOException</code> if communications error
     */
    public synchronized boolean TWGOpenLink(String hostname, String userid,
        String password) throws UnknownHostException, IOException {
        if(!is_inited)
            throw new IOException("TWGOpenLink: TCPIPDataLink not initialized");
        if(is_server_sock)
            throw new IOException("TWGOpenLink: TCPIPDataLink not console link");
	/* If socket already open, close it                                 */
        if(sock != null) {
            sock.close();
            sock = null;
        }
        /* Create socket and connect to the server                          */
        sock = new Socket(hostname, portnum);
        /* Return success                                                   */
        return true;
    }
    /**
     * Open connection to server, using given host/target name.
     * Authentication code not used.
     *
     * @param   hostname - string for identifying target server
     * @param   authcode - long for authentication code
     * @return  true if connection is successful, or false if not
     * @exception       <code>UnknownHostException</code> if bad hostname
     * @exception       <code>IOException</code> if communications error
     */
    public synchronized boolean TWGOpenLink(String hostname, long authcode)
        throws UnknownHostException, IOException {
        if(!is_inited)
            throw new IOException("TWGOpenLink: TCPIPDataLink not initialized");
        if(is_server_sock)
            throw new IOException("TWGOpenLink: TCPIPDataLink not console link");
	/* If socket already open, close it                                 */
        if(sock != null) {
            sock.close();
            sock = null;
        }
        /* Create socket and connect to the server                          */
        sock = new Socket(hostname, portnum);
        /* Return success                                                   */
        return true;
    }
    /**
     * Write data to opened connection.  Write call only returns when whole
     * data block has been written.  Data block is sent as a message which
     * will be read by a single <code>TWGReadData</code> call on the receiving end of
     * the link.
     *
     * @param b - source buffer
     * @param off - index of first byte to write from buffer
     * @param len - number of bytes to write from buffer
     * @exception <code>IOException</code> - if error during write operation
     */
    public void TWGWriteData(byte[] b, int off, int len)
        throws IOException {
        Socket s = sock;
        if(s != null) {
                OutputStream out = s.getOutputStream();
		/* Write length of message (7-bit length encoded)           */
                if(len < 128) {
                    out.write(len);
                }
                else if(len < 16384) {
                    out.write((len & 0x7F) | 0x80);
                    out.write((len >> 7) & 0x7F);
                }
                else if(len < (2 << 21)) {
                    out.write((len & 0x7F) | 0x80);
                    out.write(((len >> 7) & 0x7F) | 0x80);
                    out.write((len >> 14) & 0x7F);
                }
                else {
                    out.write((len & 0x7F) | 0x80);
                    out.write(((len >> 7) & 0x7F) | 0x80);
                    out.write(((len >> 14) & 0x7F) | 0x80);
                    out.write((len >> 21) & 0x7F);
                }
                /* Write data for header                                    */
                out.write(b, off, len);
        }
        else {
                throw new IOException();
        }
    }
    /**
     * Read data from opened connection.  Reads next message which was
     * written, as a unit.  Whole message is read, and as much as possible
     * is returned in the provided buffer range.  The actual length of
     * the message read is always returned.
     *
     * @param b - destination buffer
     * @param off - index of first byte to read into buffer
     * @param len - number of bytes to read into buffer
     * @return number of bytes in read message: if more than 'len', data
     * was lost due to insufficient buffer space.
     * @exception <code>IOException</code> - if error during read operation
     */
    public int  TWGReadData(byte[] b, int off, int len) throws IOException {
        int msg_len = 0;
	Socket s = sock;
        if(s != null) {
            InputStream inp = s.getInputStream();
            int l;
            int shft = 0;
	    /* First, read the length of the next message                   */
            do {
                l = inp.read();
                if(l == -1)
                    throw new IOException();
                msg_len |= ((l & 0x7F) << shft);
                shft += 7;
            } while((l & 0x80) == 0x80);
            /* Next, read the number of data bytes indicated                */
            int read_to_go = msg_len;
            int	cur_off = off;
	    if(msg_len > len) {         /* If too much data to save, only   */
	        read_to_go = len;       /* read what we can fit	            */
	    }
            int skip_to_go = msg_len - read_to_go;	/* Skip rest	    */
	    /* Read as much of message as possible			    */
            while(read_to_go > 0) {
                /* Try to read all remaining data			    */
                l = inp.read(b, cur_off, read_to_go);
                if(l == -1) 
                    throw new IOException();
                cur_off += l;
                read_to_go -= l;
            }
	    /* Skip any extra that we couldn't fit			    */
            while(skip_to_go > 0) {
                /* Try to read all remaining data			    */
                l = (int)inp.skip(skip_to_go);
	        if(l == -1) 
                    throw new IOException();
                skip_to_go -= l;
            }
        }
        else {
            throw new IOException();
        }
        return msg_len;
    }
    /**
     * Close connection to server or console.
     */
    public void TWGCloseLink() throws IOException {
        if(is_server_sock) {
            if(server_sock != null) {
                server_sock.close();
                server_sock = null;
            }
        }
        else {
            if(sock != null) {
                sock.close();
                sock = null;
            }
        }
    }
    /**
     * Initialize a server <code>TCPIPDataLink</code> instance.  Passes in a parameter
     * string which can either be null or include a decimal number providing
     * an override to the default port number (or * for no override).  
     * @param init_str - initialization string from link configuration
     * @exception  <code>IOException</code> if communications error
     * @exception  <code>TWGInitParmException</code> if bad initialization string
     * @exception  <code>TWGLinkNotSupportedException</code> if link type not supported
     */
    public void TWGInitializeServerLink(String init_str) throws IOException,
        TWGInitParmException, TWGLinkNotSupportedException {
        portnum = default_portnum;
        /* If we've got an initialization string, parse it		    */
        if(init_str != null) {
            String s = init_str.trim();
            if((s.length() > 0) && (!s.startsWith("*"))) {
                try {
                    portnum = Integer.parseInt(s);
                } catch(NumberFormatException n) {
                    throw new TWGInitParmException(
                        "TWGInitializeServerLink: Bad Initialization string - " + 
                        init_str);
                }
            }
        }
        if(server_sock != null) {	/* Clean up socket if already there */
            server_sock.close();
            server_sock = null;
        }
	server_sock = new ServerSocket(portnum);
        is_inited = true;	/* Mark as initialized			    */
        is_server_sock = true;  /* Is setup as server socket                */
    }
    /**
     * Listen for link from console to a server <code>TCPIPDataLink</code>.  
     * Blocks until a connection occurs, which results in the return of a 
     * <code>TCPIPDataLink</code> instance representing the server-to-console 
     * connection, or until an error occurs.  
     * @return a <code>TCPIPDataLink</code> instance for the opened session
     * @exception <code>IOException</code> on an error or when the server 
     * <code>TCPIPDataLink</code> is closed using <code>TWGCloseLink()</code>.
     */
    public TWGDataLink  TWGServerListen() throws IOException {
        if(!is_server_sock)
            throw new IOException("TWGServerListen: Not server socket"); 
        if(!is_inited)
            throw new IOException("TWGServerListen: not initialized");
	ServerSocket ss = server_sock;
        if(ss == null)
	    throw new IOException("TWGServerListen: no server socket");
        Socket s = ss.accept();
        TWGDataLink lnk = new TCPIPDataLink(s);
	return lnk;
    }
    /**
     * Return string representing host associated with link
     * @return string hostname
     */
    public String       TWGGetHostname() {
        if(sock != null) {
            return sock.getInetAddress().getHostName();
        } else if(server_sock != null) {
            return server_sock.getInetAddress().getHostName();
        } else {
            return "localhost";
        }
    }
    /**
     * Return string representing name of data link driver
     * @param loc - locale to be used to return name
     * @return string name of driver
     */
    public String       TWGGetLinkDriverName(Locale loc) {
        return "TCP/IP";        /* Same for all locales                     */
    }
}
