/**
 *
 *   Description:            The TWGMSCSObjectBrowser class is the
 *                           base class for displaying a cluster
 *                           object in the instance pane of the Cluster
 *                           Browser.
 *
 *   Author:                 Nathan Fritze (with influences by Carolyn Lay)
 *
 */

package com.tivoli.twg.mscs;


import java.io.*;
import java.awt.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.*;

import com.tivoli.uif.core.*;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.console.TWGConsoleColor;
import com.tivoli.twg.guilibs.*;
import javax.swing.table.*;
import com.tivoli.twg.libs.*;


public class TWGMSCSObjectBrowser {
    private int                     status=0;
    private JPanel                  mainPanel=null;
    private String                  statustext=null;
    private TWGMSCSClusterObject    curSelection=null;
    private TWGMSCSClusterObject    curObject=null;
    private TWGTable                table=null;
    private WindowAttributes        wndAttribs=null;

    public TWGMSCSObjectBrowser(TWGMSCSClusterObject newObject) {
        // we need to build the main panel here, so it's valid for any
        // subpanels to attach to
        mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        mainPanel.setAlignmentX(JPanel.LEFT_ALIGNMENT);
        mainPanel.setAlignmentY(JPanel.TOP_ALIGNMENT);
        mainPanel.setBackground(TWGConsoleColor.getWindowColor());

        // call setClusterObject to rebuild the subpanel for display, etc
        setClusterObject(newObject);
    }

    /**
     * Construct the window panel that is to be displayed. 
     */
    public JPanel getDisplayWindow() {
        return mainPanel;
    }

    /**
     * Provide the instance browser's current status as one of the 
     * TWGMSCSConstants status values.
     */
    public int getStatus() {
        return status;
    }

    /**
     * Set the instance browser status to the new value provided.
     * @param newStatus - a TWGMSCSConstants status value indicating the
     *                    new status of the instance browser.
     */
    public void setStatus( int newStatus ) {
        status=newStatus;
    }

    /**
     * Provide a display string describing instance browser's current status.
     */
    public String getStatusText() {
        return statustext;
    }

    /**
     * Update the display string describing instance browser's current 
     * status.
     * @param newStatusText - the string that reflects the new status
     *                         of the instance browser.
     */
    public void setStatusText( String newStatusText ) {
        statustext=newStatusText;
    }

    /**
     * Provide the class whose instances the instance browser is 
     * currently displaying.
     */
    public TWGMSCSClusterObject getClusterObject() {
        return curObject;
    }

    /**
     * Update the class whose instance the instance browser will now
     * be displaying and update the display to match.
     * @param newClusterObject - the new cluster object to display
     */
    public void setClusterObject( TWGMSCSClusterObject newClusterObject ) {
        // mark the new object to display
        curObject=newClusterObject;

        // nothing should now be selected
        curSelection=null;

        // need to rebuild the main panel here.
        mainPanel.removeAll();

        if (newClusterObject != null) {
            boolean useChild=true;
            int     cols, rows, i, j, popcount, itemsAdded;
            TWGMSCSClusterObject rowObject;

            // build a table to display
            rows=curObject.getClusterChildCount();
            if (rows==0) {
                useChild=false;
                rows=1;
            }
            if (useChild) {
                cols=curObject.getChildColumns();
            } else {
                cols=curObject.getColumns();
            }

            // initialize my rows and columns
            String  initData[][] = new String[1][cols];
            String  headers[] = new String[cols];

            // build dummy data, just for now
            for (i=0; i<cols; i++) {
                initData[0][i]=new String("Dummy");
                if (useChild) {
                    headers[i]=curObject.getChildColumnHeader(i);
                } else {
                    headers[i]=curObject.getColumnHeader(i);
                }
            }

            // build the table
            table = new TWGTable(initData, headers);
            table.setShowGrid(false);
//            table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
            table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
            for (i=0;i<table.getColumnCount();i++) {
                table.setJLabelColumn(i);
                table.setBeveledColumnHeader(i, JLabel.CENTER);
            }
            // do not let the user hide columns
            table.setUserCanHideColumns(false);
            // do not let the user re-order headers
            table.getTableHeader().setReorderingAllowed(false);
            table.setRowSelectionAllowed(true);
            table.setColumnSelectionAllowed(false);

            // make the table single select
            DefaultListSelectionModel dlsm=new DefaultListSelectionModel();
            dlsm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            table.setSelectionModel(dlsm);

            // remove the dummy strings we put in a moment ago
            ((TWGTableModel)table.getModel()).removeRow(0);

            // add my child rows
            for (i=0;i<rows;i++) {
                String  rowdata[] = new String[cols];
                // one column at a time
                for (j=0;j<cols;j++) {
                    if (useChild) {
                        rowdata[j]=curObject.getChildColumnString(i,j);
                    } else {
                        rowdata[j]=curObject.getColumnString(j);
                    }
                }
                // find what object this row is representing
                if (useChild) {
                    rowObject=curObject.getChild(i);
                } else {
                    rowObject=curObject;
                }
                // add the row using the object as object data
                table.addRow(rowdata, rowObject);
                // now add the popups
                 popcount=rowObject.getPopupItemCount();
                 itemsAdded=0;
                 for (j=0;j<popcount;j++) {
                     if (rowObject.getPopupItem(j)==null) {
                         // add a separator here, assuming this isn't the very first slot
                         if (itemsAdded>0) {
                             table.addMenuSeparatorToRow(itemsAdded-1, rowObject);
                         }
                     } else {
                         table.addPopupItemToRow(rowObject.getPopupItem(j), rowObject);
                         itemsAdded++;
                     }
                 }
            }
            table.lockTable(); // D#10475 - set fields in table non-editable

            // now add the table to the main panel
            mainPanel.setLayout(new BorderLayout());
            JScrollPane sp=table.getScrollableView();
//            sp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            sp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            mainPanel.add(sp, BorderLayout.CENTER);
        }
    }

    /**
     * Provide the currently-selected instance.
     */
    public TWGMSCSClusterObject getSelectedClusterObject() {
        return curSelection;
    }

    /**
     * Set the currently-selected instance to the instance provided.
     * @param newMSCSinstance - the instance that the instance browser
     *                         should highlight and update the     
     *                         properties browser to display.
     */
    public void setSelectedClusterObject( TWGMSCSClusterObject newClusterObject ) {
        // save the new selection
        curSelection=newClusterObject;

        // select the new object!
    }

    /**
     * Set the Window Attributes.
     */
    public void setWindowAttributes( WindowAttributes wa ) {
        // save the new attibutes
        wndAttribs=wa;

        // update the window with the new attributes (nkf - fix!)
    }

    /**
     * Return the Window Attributes.
     */
    public WindowAttributes getWindowAttributes() {
        return wndAttribs;
    }

    /**
     * Register to be a propertyChange listener.
     */
    public void addPropertyChangeListener( PropertyChangeListener listener ) {
        ; // nkf - fix!
    }

    /**
     * De-register as a propertyChange listener.
     */
    public void removePropertyChangeListener( PropertyChangeListener listener ) {
        ; // nkf - fix!
    }
}
