/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2003 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

package com.tivoli.twg.netipc;

import java.net.*;
import java.util.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.*;
import java.io.*;
import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;

/**
 * Each instance of this TWGDatagramSocket subclass provides a
 * UDP datagram socket which receives datagrams on a specific port.
 *
 * @author John Pechacek
 * @version 1.6 - 8/30/99
 */

public class TWGUDPSocket extends TWGDatagramSocket {
   public static final String DRIVER_NAME = "UDP";
   public static final int DEFAULT_PORT_NUMBER = (int)0x37A7;
   public static final int DISCARD_PORT = 9;
   //public static final int NETFINITY_PORT_NUMBER = (int)0x36A7;
   public static final String GENERAL_BROADCAST_ADDRESS = "255.255.255.255";
   public static final String DEF_MULTICAST_ADDR = "224.0.1.118";
   public static final String PROP_EXT = ".prop";
   private IntKeyTable addresses = new IntKeyTable();
   private MulticastSocket sock;
   private int portnum;
   private String multicast_grp;
   private byte last_multicast_ttl;
   private int sock_rcv_err_cnt = 0;
   private int sock_clsd_err_cnt = 0;

   private static final String PROPERTY_KEY_MAX_PKT_LEN             = "MAX_PKT_LEN";
   private static final String PROPERTY_KEY_WINDOW_LEN              = "WINDOW_LEN";
   private static final String PROPERTY_KEY_MAX_DATA_RETRIES        = "MAX_DATA_RETRIES";
   private static final String PROPERTY_KEY_MAX_SEND_ACK_RETRIES    = "MAX_SEND_ACK_RETRIES";
   private static final String PROPERTY_KEY_MAX_RECV_ACK_RETRIES    = "MAX_RECV_ACK_RETRIES";
   private static final String PROPERTY_KEY_MAX_SEND_INIT_RETRIES   = "MAX_SEND_INIT_RETRIES";
   private static final String PROPERTY_KEY_SEND_INIT_TIMEOUT       = "SEND_INIT_TIMEOUT";
   private static final String PROPERTY_KEY_SIMPLE_SEND_TIMEOUT     = "SIMPLE_SEND_TIMEOUT";
   private static final String PROPERTY_KEY_DATA_ACK_TIMEOUT        = "DATA_ACK_TIMEOUT";
   private static final String PROPERTY_KEY_DATA_RECV_TIMEOUT       = "DATA_RECV_TIMEOUT";
   private static final String PROPERTY_KEY_RECV_ACK_TIMEOUT        = "RECV_ACK_TIMEOUT";
   private static final String PROPERTY_KEY_SEND_DONE_TIMEOUT       = "SEND_DONE_TIMEOUT";
   private static final String PROPERTY_KEY_IS_RELIABLE             = "IS_RELIABLE";
   private static final String PROPERTY_KEY_MAX_XFER_WAIT_TIME      = "MAX_XFER_WAIT_TIME";
 
    
    /**
     * Default constructor
     */
    public TWGUDPSocket() {
        
        try {
            FileInputStream fis = new FileInputStream(TWGEnvironment.getDataDirectory() + File.separator + "tcpip.ini");
            Properties prop = new Properties();
            if (fis != null)
            {
                prop.load(fis);

                // @22255 -- Removed sizing options

                String value = prop.getProperty( PROPERTY_KEY_MAX_DATA_RETRIES );
                if (value != null)
                {
                    try {
                        max_data_retries = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket MAX_DATA_RETRIES="+ max_data_retries);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for MAX_DATA_RETRIES:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_MAX_SEND_ACK_RETRIES );
                if (value != null)
                {
                    try {
                        max_send_ack_retries = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket MAX_SEND_ACK_RETRIES="+ max_send_ack_retries);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for MAX_SEND_ACK_RETRIES:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_MAX_RECV_ACK_RETRIES );
                if (value != null)
                {
                    try {
                        max_recv_ack_retries = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket MAX_RECV_ACK_RETRIES="+ max_recv_ack_retries);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for MAX_RECV_ACK_RETRIES:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_MAX_SEND_INIT_RETRIES );
                if (value != null)
                {
                    try {
                        max_send_init_retries = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket MAX_SEND_INIT_RETRIES="+ max_send_init_retries);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for MAX_SEND_INIT_RETRIES:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_SEND_INIT_TIMEOUT );
                if (value != null)
                {
                    try {
                        send_init_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket SEND_INIT_TIMEOUT="+ send_init_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for SEND_INIT_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_SIMPLE_SEND_TIMEOUT );
                if (value != null)
                {
                    try {
                        simple_send_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket SIMPLE_SEND_TIMEOUT="+ simple_send_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for SIMPLE_SEND_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_DATA_ACK_TIMEOUT );
                if (value != null)
                {
                    try {
                        data_ack_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket DATA_ACK_TIMEOUT="+ data_ack_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for DATA_ACK_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_DATA_RECV_TIMEOUT );
                if (value != null)
                {
                    try {
                        data_recv_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket DATA_RECV_TIMEOUT="+ data_recv_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for DATA_RECV_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_RECV_ACK_TIMEOUT );
                if (value != null)
                {
                    try {
                        recv_ack_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket RECV_ACK_TIMEOUT="+ recv_ack_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for RECV_ACK_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_SEND_DONE_TIMEOUT );
                if (value != null)
                {
                    try {
                        send_done_timeout = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket SEND_DONE_TIMEOUT="+ send_done_timeout);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for SEND_DONE_TIMEOUT:" + value);
                    }
                }
                value = prop.getProperty( PROPERTY_KEY_IS_RELIABLE );
                if (value != null)
                {
                    is_reliable = Boolean.valueOf(value).booleanValue();
                    TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket IS_RELIABLE="+ is_reliable);
                }
                value = prop.getProperty( PROPERTY_KEY_MAX_XFER_WAIT_TIME );
                if (value != null)
                {
                    try {
                        max_xfer_wait_time = Integer.parseInt(value);
                        TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket MAX_XFER_WAIT_TIME="+ max_xfer_wait_time);
                    } catch (NumberFormatException nfe) {
                        TWGRas.error(TWGRas.NETIPC,"TWGUDPSocket Invalid value for MAX_XFER_WAIT_TIME:" + value);
                    }
                }
            }
        } catch (FileNotFoundException fnf) {
            TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket Using default tcpip settings.");
        } catch (IOException ioe) {
            TWGRas.debug(TWGRas.NETIPC,"TWGUDPSocket Unable to open tcpip.ini file.");
        }
    }


    /*
     * Start a UDP datagram socket
     * @param name opaque network protocol name used for TWGNetAddress instances
     *     created by this socket - name is used to form IPC path
     * @param init_str initialization string : consists of port number (*=unchanged)
     *    followed by multicast group (*=unchanged)
     */
    public void Start(String name, String init_str)
    throws SocketException, TWGIPCTransportException {
       super.Start(name, init_str);
       portnum = DEFAULT_PORT_NUMBER;
       multicast_grp = DEF_MULTICAST_ADDR;
       /* If we've got an initialization string, parse it		    */
       if(init_str != null) {
           StringTokenizer st = new StringTokenizer(init_str);
           try {
               String port_str = st.nextToken();
               if(port_str.equals("*")) {
                   /* Leave at default					    */
               } else {
                   try {
                       portnum = Integer.parseInt(port_str);
                   } catch(NumberFormatException n) {
                       throw new TWGIPCTransportException(
                           "Invalid TWGUDPSocket initialization string - " +
                           init_str);
                   }
               }
               String multicast_addr = st.nextToken();
               try {
                   InetAddress multiaddr = InetAddress.getByName(multicast_addr);
                   if(multiaddr.isMulticastAddress() == true) {
                       multicast_grp = multiaddr.getHostAddress();
                   } else {
                       throw new TWGIPCTransportException(
                           "Invalid TWGUDPSocket initialization string - " +
                           init_str + " - bad multicast group");
                   }
               } catch (UnknownHostException uhx) {
                   throw new TWGIPCTransportException(
                       "Invalid TWGUDPSocket initialization string - " +
                       init_str + " - bad multicast group");
               } catch (IllegalArgumentException iae) {
                   throw new TWGIPCTransportException(
                       "Invalid TWGUDPSocket initialization string - " +
                       init_str + " - bad multicast group");
               }
           } catch (NoSuchElementException nsex) {
           }
       }
       try {
           sock = new MulticastSocket(portnum);
       } catch (IOException iox) {
           throw new TWGIPCTransportException("Error creating UDPSocket - " + iox);
       }
       try {
           InetAddress multi_grp = InetAddress.getByName(multicast_grp);
           sock.joinGroup(multi_grp);
       } catch (IOException iox) {
           TWGRas.error(TWGRas.NETIPC, "Error joining multicast group - " + multicast_grp + ": " + iox);
       } catch (IllegalArgumentException iae) {
           TWGRas.error(TWGRas.NETIPC, "Invalid multicast group - " + multicast_grp);
       }
    }
    /**
     * Send datagram over socket
     *
     * @param target_addr target address
     * @param b source buffer
     * @param off index of first byte to write from buffer
     * @param len number of bytes to write from buffer
     * @exception IOException if error during write operation
     */
    public void SendDatagram(TWGNetAddress target_addr, byte[] b, int off, int len)
    throws IOException {
       byte[] sendbuf = new byte[len];
       System.arraycopy(b, off, sendbuf, 0, len);
       SendDatagram(target_addr, sendbuf, len);
    }
    /**
     * Send datagram over socket
     *
     * @param target_addr target address
     * @param b source buffer
     * @param len number of bytes to write from buffer
     * @exception IOException if error during write operation
     */
    public void SendDatagram(TWGNetAddress target_addr, byte[] b, int len)
    throws IOException {
       final MulticastSocket socket = sock;
       if (socket == null) {
          throw new IOException("UDP socket not ready");
       }
       InetAddress inet_addr = ((TWGIPAddress)target_addr).getAddress();
       int port = ((TWGIPAddress)target_addr).getPort();
       final DatagramPacket sendPacket = new DatagramPacket(b, len, inet_addr, port);

       if (TWGRas.isTraceNETIPC && TWGRas.isTraceHigh) {
          TWGRas.debugH(TWGRas.NETIPC, "[SEND] " + getProtocolName() + "::" + sendPacket.getAddress().getHostAddress() 
                       + " " + TWGDatagram.formatDatagramDebug(sendPacket.getData(), 0)) ;
       }

       try {
           AccessController.doPrivileged(new PrivilegedExceptionAction() {
               public Object run() throws IOException {
                   socket.send(sendPacket);
                   return ( null );
               }
           });
       } catch (PrivilegedActionException pax) {
           throw (IOException)pax.getException();
       }
    }

    /**
     * Broadcast datagram to all configured broadcast addresses
     *
     * @param b - source buffer
     * @param len - number of bytes to write from buffer
     * @exception IOException - if error during write operation
     */
    public void BroadcastDatagram(byte[] b, int len)
    throws IOException {
       BroadcastDatagram(b, len, portnum);
    }

    public void BroadcastDatagramOnDiscardPort(byte[] b, int len)
    throws IOException {
       BroadcastDatagram(b, len, DISCARD_PORT);
    }

    public void BroadcastDatagram(byte[] b, int len, int port)
    throws IOException {
       if (sock == null) {
          throw new IOException("UDP socket not ready");
       }
       /* Load TCPIP broadcast properties */
       NetDriverTCPIPProps driverprops = new NetDriverTCPIPProps(protocol_name + PROP_EXT);
       driverprops.load();
       String val;
       int i;

       /* Send to unicast addresses */
       for (i=0; i < driverprops.uni_addresses.size(); i++) {
          val = (String)driverprops.uni_addresses.elementAt(i);
          StringTokenizer st = new StringTokenizer(val, "-");
          String addr1_string = st.nextToken().trim();
          String addr2_string;
          if (st.hasMoreTokens()) {
             addr2_string = st.nextToken().trim();
          } else {
             addr2_string = addr1_string;
          }
          try {
              int addr1 = IpAddrUtil.getIpAddr(addr1_string);
              int addr2 = IpAddrUtil.getIpAddr(addr2_string);
              if (addr2 >= addr1) {
                 TWGRas.debug(TWGRas.NETIPC,"Sending to unicast address range: " + addr1_string + " - " + addr2_string);
                 for (int addr=addr1; addr <= addr2; addr++) {
                    String addrstr = IpAddrUtil.ipAddrToString(addr);
                    TWGRas.debug(TWGRas.NETIPC,"Sending datagram to unicast address " + addrstr);
                    SendDatagram(addrstr, b, len, port);
                 }
             }
          } catch (NumberFormatException nfx) {	/* Handle bad address formats			    */
             TWGRas.error(TWGRas.NETIPC, "Error: unicast address badly formatted: val=" + val + ", addr1_string=" +
                 addr1_string + ", addr2_string=" + addr2_string );
          }
       }

       /* Send as relay datagram to relay node addresses */
       /* First build relay datagram */
       int rdglen = TWGDatagram.DGRAM_SIZE_REMOTE_REQ + len;
       byte[] rdg = new byte[rdglen];
       rdg[0] = TWGDatagram.DGRAM_REMOTE_REQ;
       rdg[1] = (port == DISCARD_PORT) ? TWGDatagram.DGRAM_RMT_WAKE : TWGDatagram.DGRAM_RMT_FORWARD;
       IntelByteBuffer.WriteUSHORT(rdg, 0, TWGDatagram.DGRAM_RMT_NAME_LEN_OFF);
       System.arraycopy(b, 0, rdg, TWGDatagram.DGRAM_RMT_NAME_OFF, len);
       /* Then unicast the relay datagram to each relay node address */
       for (i=0; i < driverprops.relay_addresses.size(); i++) {
          val = (String)driverprops.relay_addresses.elementAt(i);
          StringTokenizer st = new StringTokenizer(val, "-");
          String addr1_string = st.nextToken().trim();
          String addr2_string;
          if (st.hasMoreTokens()) {
             addr2_string = st.nextToken().trim();
          } else {
             addr2_string = addr1_string;
          }
          try {
              int addr1 = IpAddrUtil.getIpAddr(addr1_string);
              int addr2 = IpAddrUtil.getIpAddr(addr2_string);
              if (addr2 >= addr1) {
                 TWGRas.debug(TWGRas.NETIPC,"Sending to relay address range: " + addr1_string + " - " + addr2_string);
                 for (int addr=addr1; addr <= addr2; addr++) {
                    String addrstr = IpAddrUtil.ipAddrToString(addr);
                    TWGRas.debug(TWGRas.NETIPC,"Sending datagram to relay address " + addrstr);
                    SendDatagram(addrstr, rdg, rdglen, portnum); // always send to regular port
                 }
             }
          } catch (NumberFormatException nfx) {	/* Handle bad address formats			    */
             TWGRas.error(TWGRas.NETIPC, "Error: relay address badly formatted: val=" + val + ", addr1_string=" +
                 addr1_string + ", addr2_string=" + addr2_string );
          }
       }

       /* Send to general broadcast address if enabled */
       if (driverprops.broadGeneral) {
          TWGRas.debug(TWGRas.NETIPC,"Sending datagram to general broadcast address " + GENERAL_BROADCAST_ADDRESS);
          SendDatagram(GENERAL_BROADCAST_ADDRESS, b, len, port);
       }

       /* Send to directed broadcast addresses */
       for (i=0; i < driverprops.broad_addresses.size(); i++) {
         val = (String)driverprops.broad_addresses.elementAt(i);
         int space_index = val.indexOf(" ");
         String inet_addr_string = val.substring(0, space_index);
         String mask_addr_string = val.substring(space_index+1);
         byte[] inet_addr = InetAddress.getByName(inet_addr_string).getAddress();
         byte[] mask_addr = InetAddress.getByName(mask_addr_string).getAddress();
         inet_addr[0] &= mask_addr[0];
         inet_addr[1] &= mask_addr[1];
         inet_addr[2] &= mask_addr[2];
         inet_addr[3] &= mask_addr[3];
         inet_addr[0] |= (mask_addr[0] ^ (byte)0xFF);
         inet_addr[1] |= (mask_addr[1] ^ (byte)0xFF);
         inet_addr[2] |= (mask_addr[2] ^ (byte)0xFF);
         inet_addr[3] |= (mask_addr[3] ^ (byte)0xFF);
         val = Integer.toString(inet_addr[0] & 0xFF) + "." +
            Integer.toString(inet_addr[1] & 0xFF) + "." +
            Integer.toString(inet_addr[2] & 0xFF) + "." +
            Integer.toString(inet_addr[3] & 0xFF);
         TWGRas.debug(TWGRas.NETIPC,"Sending datagram to directed broadcast address " + val);
         SendDatagram(val, b, len, port);
       }

       /* Send to multicast addresses */
       for (i=0; i < driverprops.multicast_addresses.size(); i++) {
         String multicast_addr = (String)driverprops.multicast_addresses.elementAt(i);
         if(last_multicast_ttl != (byte)driverprops.multicast_ttl) {
            try {
               sock.setTimeToLive(driverprops.multicast_ttl);
            } catch (IOException e) {
               // Catch OS/400 V4R4 TTL bug
               TWGRas.error(TWGRas.NETIPC,"Error setting TTL");
            }
            last_multicast_ttl = (byte)driverprops.multicast_ttl;
         }
         TWGRas.debug(TWGRas.NETIPC,"Sending datagram to multicast address " + multicast_addr);
         SendDatagram(multicast_addr, b, len, port);
       }
    }
    private void SendDatagram(String val, byte[] b, int len, int port)
    throws IOException {
       TWGIPAddress addr;
       try {
          addr = new TWGIPAddress(InetAddress.getByName(val), port, this);
          SendDatagram(addr, b, len);
       } catch (UnknownHostException e) {
          TWGRas.error(TWGRas.NETIPC,"Error sending datagram to " + val + ": " + e);
       } catch (IOException e) {
          TWGRas.error(TWGRas.NETIPC,"Error sending datagram to " + val + ": " + e);
       } catch (IllegalArgumentException iae) {
           TWGRas.error(TWGRas.NETIPC, "Error sending datagram to " + val + ": " + iae);
       }
    }
    /**
     * Receive datagram from socket
     *
     * @return TWGReceivedDatagram - datagram received
     * @exception IOException - if error during receive operation
     */
    public TWGReceivedDatagram ReceiveDatagram()
    throws IOException {
       byte[] inbuf = new byte[TWGDatagramSocket.MAX_DATAGRAM_SIZE];
       while (true) {
          try {
             DatagramPacket inPacket = new DatagramPacket(inbuf, TWGDatagramSocket.MAX_DATAGRAM_SIZE);
             if (sock == null) {
                 throw new IOException("UDP socket not ready");
             }
             sock.receive(inPacket);
             sock_rcv_err_cnt = 0;
             sock_clsd_err_cnt = 0;
             if (TWGRas.isTraceNETIPC && TWGRas.isTraceHigh) {
                TWGRas.debugH(TWGRas.NETIPC, "[RECV] " + getProtocolName() +"::" + inPacket.getAddress().getHostAddress()
                             + " " + TWGDatagram.formatDatagramDebug( inPacket.getData(), 0 ) );
             }
             return new TWGReceivedDatagram(GetAddress(inPacket.getAddress(), inPacket.getPort()),
                                            inPacket.getData(), inPacket.getLength());
          } catch (IOException e) {
             if (e.getMessage().startsWith("Connection refused") && (sock_rcv_err_cnt < 50)) {
                sock_rcv_err_cnt++;
                TWGRas.debugH(TWGRas.NETIPC,"Ignored exception due to suspected JRE bug (sock_rcv_err_cnt="+sock_rcv_err_cnt+") : " + e);
                /* Wait half a second before retrying */
                Object tmpwait = new Object();
                synchronized (tmpwait) {
                   try {
                      tmpwait.wait(500);
                   } catch (InterruptedException e2) {
                   }
                }
             }
             else if (e.getMessage().startsWith("socket closed") && (sock_clsd_err_cnt < 50)) {
                 // @22266: If a UDP packet is sent to a host with no process bound to the
                 // destination port, linux returns an ICMP message destination unreachable.
                 // This causes a Socket Closed exception to be generated in Java.  We need
                 // to ignore this exception and continue to process receives.
                 sock_clsd_err_cnt++;
                 TWGRas.debugH(TWGRas.NETIPC,"Received socket closed ICMP message; cnt = " + sock_clsd_err_cnt);
             }
             else {
                throw e;
             }
          }
       }
    }
    /**
     * Resolve InetAddress instance to an existing or new TWGIPAddress
     */
    private TWGIPAddress GetAddress(InetAddress inetAddr, int port) {
       TWGIPAddress netAddress = (TWGIPAddress)addresses.FindObject(inetAddr.hashCode());
       if (netAddress == null) {
          netAddress = new TWGIPAddress(inetAddr,
                                        port,
                                        this);
          addresses.InsertObject(inetAddr.hashCode(), netAddress);
       }
       return netAddress;
    }
    /**
     * Close socket
     */
    public void Close() throws IOException {
       if (sock!= null) {
           sock.close();
           sock = null;
           //D14257 Fix build break
           try
              {
              Thread.currentThread().sleep(1000);
              }
           catch (Exception e)
              {
              }
           addresses.EmptyTable();
       }
    }
    /**
     * Return normalized format of the specified address name
     * @return String in format "%d.%d.%d.%d"
     * @exception TWGNetAddressException if specified address name not resolved
     */
    public String NormalizeAddressName(String name)
    throws TWGNetAddressException
    {
       InetAddress addr;
       try {
          addr = InetAddress.getByName(name);
       } catch (UnknownHostException uhe) {
          throw new TWGNetAddressException("Failed to normalize address name " + name);
       } catch (IllegalArgumentException iae) {
          throw new TWGNetAddressException("Invalid address name " + name);
       }
       //int slash_offset = addrString.indexOf("/");
       //if (slash_offset > 0) {
       //   addrstring = addrstring.substring(0, slash_offset);
       //}
       return addr.getHostAddress();
    }
    /**
     * Generate TWGNetAddress instance from specified address name
     * @return TWGNetAddress instance from specified name or null if
     *     name was not resolved
     */
    public TWGNetAddress GetNetAddress(String address_name)
    throws TWGNetAddressException
    {
       InetAddress addr;
       try {
          addr = InetAddress.getByName(address_name);
       } catch (UnknownHostException uhe) {
          throw new TWGNetAddressException("Failed to resolve address for name " + address_name);
       } catch (IllegalArgumentException iae) {
          throw new TWGNetAddressException("Invalid address name " + address_name);
       }
       return GetAddress(addr, portnum);
    }
    /**
     * Return string representing name of datagram socket driver
     * @return string name of driver
     */
    public String GetDriverName() {
        return DRIVER_NAME;
    }


}
