#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef _Windows
  #include <windows.h>
  #include "nfwincom.hpp"
#endif

#include "cmd.hpp"
#include "svcnode.hpp"
#include "cp_conv.hpp"
#include "sysobjs.hpp"
#include "bobcbase.hpp"

FILE *debugOut;

void debug(char *msg) {
    if (debugOut) {
       fprintf(debugOut, "%s\n", msg);
       fflush(debugOut);
    }
}

class BobCoSvcNode : public ServiceNode {
public:
   /* Constructor : just create service node with needed name          */
   BobCoSvcNode() : ServiceNode(BOBCO_SVCNODE_NAME, FALSE) {};

   /* Handle all received commands                                     */
   virtual BOOL CommandReceived(Command &);

   /* Handle request for list of local fixed drives                    */
   void    HandleGetDrives(Command &cmd);

   /* Handle request for contents of a directory                       */
   void    HandleGetDir(Command &cmd);
};

/****************************************************************************/
/* Process any received commands, and route to needed handlers              */
/****************************************************************************/
BOOL    BobCoSvcNode::CommandReceived(Command &cmd) {
   char msg[100];
   sprintf(msg, "Command received: %08x", cmd.CommandCode());
   debug(msg);
   switch(cmd.CommandCode()) {
#ifdef __NETWARE_386__
   case NOVELL_KILL_SERVICE:
      return(FALSE); // This will cause the service node to
                     // close, thus returning from the
                     // ProcessCommands call in main(), which
                     // will kill the service
      break;
#endif
   case BOBCO_GET_LOCAL_FIXED_DRIVES:
      debug("Calling HandleGetDrives()");
      HandleGetDrives(cmd);
      break;

   case BOBCO_GET_DIRECTORY_CONTENTS:
      debug("Calling HandleGetDir()");
      HandleGetDir(cmd);
      break;

   default:
      debug("Unrecognized command code");
      cmd.SetReturnCode(RETURN_UNRECOGNIZED_COMMAND_CODE);
      break;
   }
   return TRUE;
}

/****************************************************************************/
/* This method gets the list of logical drive names, and for each one that  */
/* is a local fixed drive, adds the drive name as an output parameter (in   */
/* compressed Unicode) to the command that was passed in.                   */
/****************************************************************************/
void BobCoSvcNode::HandleGetDrives(Command &cmd) {
   char buffer[1000];
   char compUnicode[100];
   char *dName = buffer;

   GetLogicalDriveStrings(1000, buffer);
   while (*dName != 0) {
      if (GetDriveType(dName) == DRIVE_FIXED) {
         ULONG len = NFConvertLocalToCompUnicode(dName, compUnicode, 100, L'*', NULL);
         cmd.AddOutputParm(compUnicode, len);
      }
      dName += strlen(dName)+1;
   }
   cmd.SetReturnCode(RETURN_OKAY);
}

/****************************************************************************/
/* This method gets a path from the input parameter of the passed command   */
/* (in compressed Unicode), and does a find first/find next loop to         */
/* determine the contents of the specified directory.  Each subdirectory or */
/* file that is found is added along with its attributes as an output       */
/* parameter to the command that was passed in.                             */
/****************************************************************************/
void BobCoSvcNode::HandleGetDir(Command &cmd) {
   WIN32_FIND_DATA wfind;
   char buffer[1000];
   char dbbuf[1000];

   NFConvertCompUnicodeToLocal((char *)cmd.InputParm(0), buffer, 1000, L'*', NULL);
   sprintf(dbbuf, "HandleGetDir() looking for '%s'", buffer);
   debug(dbbuf);
   HANDLE h = FindFirstFile(buffer, &wfind);
   if (h != INVALID_HANDLE_VALUE) {
      FileInfo fi;
      do {
         fi.setInfo(&wfind);
         // don't include . and .. directory entries just to make it cleaner
         if (strcmp(wfind.cFileName, ".") && strcmp(wfind.cFileName,"..")) {
            cmd.AddOutputParm(&fi.cmdFileInfo, fi.fileInfoLength);
         }
      } while (FindNextFile(h, &wfind));
      FindClose(h);
      cmd.SetReturnCode(RETURN_OKAY);
   } else {
      cmd.SetReturnCode(RETURN_FIND_FAILED);
   }
}


#ifdef __NETWARE_386__
extern "C" void Cleanup()
{
        // do this to assure we clean up if we break in the middle of a discovery
        ProductRecord::CleanUpAllProducts();
        if ( svcnode ) {
           delete svcnode;
        } /* endif */
}
#endif

BobCoSvcNode *svcnode = NULL;

#ifdef _Windows
int PASCAL WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
#else
int main()
#endif
{
// debugOut = fopen("bobcbase.dbg", "a+");
   /* Register to prevent unload on user logoff                        */
   ServiceNode::RegisterAsServiceBase();

   /* Create the service node for the process                          */
   debug("Creating service node");
   svcnode = new BobCoSvcNode;
   /* If error creating service node, exit with error                  */
   if(!svcnode->Create()) {
      puts("Error creating service node!");
      return 1;
   }
   /* Once service node is created, start processing commands          */
   debug("Calling process commands");
   svcnode->ProcessCommands();
   debug("Returned from process commands");
   /* Clean up service node                                            */
   delete svcnode;
   svcnode = NULL;

   return 0;
}

/****************************************************************************/
/* This method gathers all the information about a file or directory        */
/* into a fixed structure so that it can be added as an output parameter    */
/* on a command                                                             */
/****************************************************************************/
void FileInfo::setInfo(WIN32_FIND_DATA *fd) {
   FILETIME ft;
   FileTimeToLocalFileTime(&fd->ftLastWriteTime, &ft);
   FileTimeToDosDateTime(&ft, (LPWORD)&cmdFileInfo.dosDate, (LPWORD)&cmdFileInfo.dosTime);
   cmdFileInfo.fileAttributes = fd->dwFileAttributes;
   cmdFileInfo.sizeHigh = fd->nFileSizeHigh;
   cmdFileInfo.sizeLow = fd->nFileSizeLow;
   fileInfoLength = NFConvertLocalToCompUnicode(fd->cFileName, cmdFileInfo.fileName, sizeof(cmdFileInfo.fileName), L'*', NULL);
   fileInfoLength += FIXED_CMD_FILEINFO_LENGTH;
}
