/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#include <windows.h>
#include <mon_sdk.h>

class KeyInfoBlockData;

/**
 * The KeyInfoBlock class is designed to encapsulate all of the information 
 * needed to add nodes to the monitor console, have data set for the block, 
 * and to describe the attributes of the node. The KeyInfoBlock also knows how
 * to render itself in a format that is recognized by the Monitor Agent. 
 *
 * KeyInfoBlock objects are reference counted. This has the side effect that 
 * changing the characteristics of one object also changes all reference counted
 * copies. This is a desireable side effect in the context of this sample
 */

class KeyInfoBlock
{
public:

  enum EDataType
  {
    kInvalid, kDouble, kString, kBoth
  };

  /**
   * Default Constructor
   * 
   */
  KeyInfoBlock( );

  /**
   * Constructor
   * 
   * @param displayableName This is the name displayed in the Monitor console.
   * @param programmaticName This name is used to identify blocks 
   *                         programmatically. Don't have to worry about 
   *                         translating this name to different locales.
   */
  KeyInfoBlock( char *displayableName,
                char *programmaticName );

  /**
   * Copy Constructor
   *
   * @param block KeyInfoBlock to copy
   */
  KeyInfoBlock( const KeyInfoBlock &block );
  /**
   * Destructor for KeyInfoBlock. This should be virtual if KeyInfoBlocks are to 
   * be inherited from.
   */
  ~KeyInfoBlock();
  /**
   * Used to determine if this block is in use.
   *
   * @return state of block
   */
  BOOL 
    inUse() const;
  /**
   * Set the usage state of this block.
   */
  void
    setBlockUsage( BOOL useage);

  /** 
   * Used to query the validity of data.
   *
   * @return state of data.
   */
  BOOL
    dataValid() const;
  /** 
   * Set the validity state of the data. Invalid data will not be displayed
   * by the Monitor console.
   *
   */
  void
    setDataValidity( BOOL valid );

  /**
   * Assignment operator.
   *
   */
  KeyInfoBlock 
    &operator = ( const KeyInfoBlock& node );

  /**
   * Equivalence operator.
   */
  BOOL 
    operator == ( const KeyInfoBlock& node );

  /**
   * Set the default Java Language String. If the language string is the same
   * for all KeyInfoBlock objects then this value can be set once and will be
   * used for all instances.
   */
  static void
    setDefaultJavaLangString( const char * javaLangString );
  /**
   * Returns the default Java Language String. Set to "en_US" by default.
   *
   * @return The default Java Language String. Do not free storage.
   */
  static const char *
    getDefaultJavaLangString( );
  /**
   * Set the Java Lang string. This may not be necessary if the default java
   * string is correct.
   */
  void
    setJavaLangString( const char * javaLangString );
  /**
   * Set the name of the node. This is the name that will be displayed in the 
   * monitor console. 
   *
   * This sample only supports SBCS. For i18n support this member function would
   * need to use a string class.
   */
  void
    setDisplayableName( const char * displayableName );
  /**
   * Set the programmatic name of this block. The programmatic name does not
   * need to be changed for different locales. The programmatic name could be
   * used to compare blocks, but the handle which is unique is a cheaper 
   * comparison.
   */
  void
    setProgrammaticName( const char * programmaticName );
  /**
   * Sets a string for the units of measure.
   */
  void
    setUnitsOfMeasure( const char * unitsOfMeasure );
  /**
   * Sets the format to be used for displaying the data. The format follows the
   * printf format, such as %s or %6.2f.
   */
  void
    setFormat( const char * format );
  /**
   * Sets the type for the data. The EDataType enum specifies supported formats.
   */
  void
    setDataType( EDataType dataType );
  /**
   * Sets the block as a leaf node. The user does not need to call this member
   * function. The MonitorLibrary object figures this out and sets the field
   * accordingly.
   */
  void
    setLeafNode(BOOL leafNode );
  /**
   * Gets the data type for the block.
   *
   */
  EDataType
    getDataType() const;
  /**
   * Gets the Java Lang String of the block object.
   */
  const char
    *getJavaLangString( ) const;
  /**
   * Gets the displayable name of the block object.
   */
  const char
    *getDisplayableName( ) const;
  /**
   * Gets the programmatic name of the block object.
   */
  const char
    *getProgrammaticName( ) const;
  /**
   * Gets the Units of Measure string for the block object.
   */
  const char
    *getUnitsOfMeasure( ) const;
  /**
   * Gets the string used for formatting the blocks data.
   */
  const char
    *getFormat() const;
  /**
   * Gets length of displayable name as a Compressed Unicode String.
   */
  unsigned long
    getDisplayableNameLength() const;
  /**
   * Gets the length of the programmatic name as a Compressed Unicode String.
   */
  unsigned long
    getProgrammaticNameLength() const;
  /**
   * Gets the attribute node size.
   */
  unsigned long
    getAttributeNodeSize() const;
  /**
   * Copy the block as an attribute node to the address requested. This is a
   * rendering member function used by MonitorLibrary.
   */
  void
    copyDataTo( AttributeNode *fAttributeNode );
  /**
   * Returns TRUE if the KeyInfoBlock object does not have any children.
   */
  BOOL
    isLeafNode() const;

  /**
   * Sets the base pulse. The base pulse is specified in msecs. Only leaf nodes
   * needs this information set.
   */
  void 
    setBasePulse( unsigned long basePulse );
  /**
   * Sets pulses per data.
   */
  void 
    setPulsesPerData( unsigned long pulsesPerData);
  /**
   * Set the number of data values that should be saved.
   */
  void 
    setNumberRecorded( unsigned long numberRecorded);
  /**
   * Sets the last time that data was recorded. The last time is in seconds.
   */
  void 
    setLastTime( unsigned long lastTime );

  /**
   *  Gets the base pulse. The base pulse is in msec
   */
  unsigned long
    getBasePulse( ) const;
  /**
   * Gets the last time data was recorded. The last time is in seconds.
   */
  unsigned long
    getLastTime() const;
  /**
   * Sets the double value for this KeyInfoBlock object.
   */
  void
    setDoubleValue( double value );
  /**
   * Sets the string value for this KeyInfoBlock object.
   */
  void
    setStringValue( const char *value );
  /**
   * Gets the double value for this KeyInfoBlock object. Use GetDataType to 
   * determine data type.
   */
  double
    getDoubleValue() const;
  /**
   * Gets the string value for this KeyInfoBlock object. Use GetDataType to 
   * determine data type.
   */
  const char 
    *getStringValue() const;

  /**
   * Orphans ( caller responsible to free memory ) the KeyInfoBlock as a 
   * NodeData data structure.
   *
   * @return orphaned object as a NodeData
   */
  NodeData
    *orphanNodeData( );
  /**
   * Orphans ( caller responsible to free memory ) the KeyInfoBlock as a 
   * NodeParm data structure.
   *
   * @return orphaned object as a NodeParm
   */
  NodeParms
    *orphanNodeParms( );
  /**
   * Get unique handle for this block. These KeyInfoBlocks are reference counted
   * so objects copy ( copy ctr or assignment operator ) will have the same 
   * handle.
   */
  unsigned long
    getHandle( ) const;

protected:

  private:
  KeyInfoBlockData
    *fData;
};
